CSECT   AGNEW2
C     REVISED 4/25/73
C      REVISED AGCALC ****************
 
      SUBROUTINE AGNEW2 (C44,R55,S44,B14,B17,B24,B25,B27,B34,B35,B36,
     C           B37,B84,B85,B86,B87,MSI,MRI,MLI,L66,M36,F1,F2,F3,F4,
     C           F5,F6,F7,F8,F9,F10,W1,W2,W3,W4,W5,W6,W7,W8,W9,W10,W11,
     C           W12,W13,W14,W15,W16,R1,R2,A,G,Q,H,
     C           BX5,BX4,B3Y,B2Y)
 
C   THIS SUBROUTINES CALCULATES THE A-MATRIX AND THE G-MATRIX ASSOCIATED
C   WITH THE STATE VARIABLE FORMULATION OF THE SET OF DIFFERENTIAL
C   EQUATIONS USED TO DESCRIBE THE TRANSIENT RESPONSE OF AN ELECTRICAL
C   NETWORK. THE BASIC EQUATION IS GIVEN BY
C
C                  XDOT = A*X + G*V
C
C                    WHERE
C        X - (N )         STATE VARIABLE VECTOR
C        A - (N * N)      REAL MATRIX OF CONSTANT COEFFICIENTS
C        G - (N * NS2)    REAL MATRIX OF CONSTANT COEFFICIENTS
C        V - (NS2)        VECTOR OF DRIVING SOURCES
C
C   THE ARGUMENTS IN THE CALLING SEQUENCE HAVE THE FOLLOWING DEFINITIONS
C
C     C44  - (NBC )       A VECTOR OF BRANCH CAPACITOR ELEMENTS
C     S44  - (NBC )       THE RECIPROCAL OF C44
C     R55  - (NBR )       A VECTOR OF BRANCH RESISTOR ELEMENTS
C     B14  - (NLC * NBC) CONNECTIVITY MATRIX RELATING LINK CAPACITORS TO
C                                            BRANCH CAPACITORS
C     B17  - (NLC * NBE) CONNECTIVITY MATRIX RELATING LINK CAPACITORS TO
C                                            BRANCH VOLTAGE SOURCES
C     B24  - (NLR * NBC) CONNECTIVITY MATRIX RELATING LINK RESISTORS TO
C                                            BRANCH CAPACITORS
C     B25  - (NLR * NBR) CONNECTIVITY MATRIX RELATING LINK RESISTORS TO
C                                            BRANCH RESISTORS
C     B27  - (NLR * NBE) CONNECTIVITY MATRIX RELATING LINK RESISTORS TO
C                                            BRANCH VOLTAGE SOURCES
C     B34  - (NLL * NBC) CONNECTIVITY MATRIX RELATING LINK INDUCTORS TO
C                                            BRANCH CAPACITORS
C     B35  - (NLL * NBR) CONNECTIVITY MATRIX RELATING LINK INDUCTORS TO
C                                            BRANCH RESISTORS
C     B36  - (NLL * NBL) CONNECTIVITY MATRIX RELATING LINK INDUCTORS TO
C                                            BRANCH INDUCTORS
C     B37  - (NLL * NBE) CONNECTIVITY MATRIX RELATING LINK INDUCTORS TO
C                                            BRANCH VOLTAGE SOURCES
C     B84  - (NLJ * NBC) CONNECTIVITY MATRIX RELATING LINK J-SOURCES TO
C                                            BRANCH CAPACITORS
C     B85  - (NLJ * NBR) CONNECTIVITY MATRIX RELATING LINK J-SOURCES TO
C                                            BRANCH RESISTORS
C     B86  - (NLJ * NBL) CONNECTIVITY MATRIX RELATING LINK J-SOURCES TO
C                                            BRANCH INDUCTORS
C     B87  - (NLJ * NBE) CONNECTIVITY MATRIX RELATING LINK J-SOURCES TO
C                                            BRANCH VOLTAGE SOURCES
C     M36  - (NLL * NBL) FULL MATRIX CONTAINING LINK TO BRANCH MUTUALS
C     L66  - (NBL * NBL) FULL MATRIX CONTAINING BRANCH INDUCTORS ON THE
C                        MAIN DIAGONAL & BRANCH-BRANCH MUTUALS ELSEWHERE
C     MSI  - (NLC * NLC) THE INVERSE OF MS
C WHERE MS = S11 + B14*S44*B14T
C     MRI  - (NLR * NLR) THE INVERSE OF MR
C WHERE MR = R22 + B25*R55*B25T
C     MLI  - (NLL * NLL) THE INVERSE OF ML
C WHERE ML = (  L33   ) + M36*B36T + B36*M63 + B36*(   L66    )*B36T
C       ML = (L33D+M33) + M36*B36T + B36*M63 + B36*(L66D + M66)*B36T
C
C   THE FOLLOWING FUNCTIONS ARE DEFINED AND USED INTERNALLY IN THE
C   SUBROUTINE . ..................
C     F1   - (NLL * NLL),F1 = MLI
C     F2   - (NLL * NBL),F2 = B36*L66
C     F3   - (NBC * NLC),F3 = B14T*MSI
C     F4   - (NLL * NLR),F4 = (B35*R55)*(B25T*MRI)
C     F5   - (NLL * NBR),F5 = (B35*R55)*((I - B25T*MRI)*(B25*R55))
C     F6   - (NBC * NLR),F6 = ((B14T*MSI)*(B14*S44) - I)*(B24T*MRI)
C     F7   - (NBC * NBR),F7 = (B24T*MRI)*(B25*R55)
C     F8   - (NBC * NBC),F8 = S44*((B14T*MSI)*(B14*S44) - I)
C     F9   - (NBC * NLR),F9 = S44*((B14T*MSI)*(B14*S44) - I)*(B24T*MRI)
C     F10  - (NLL * NBR),F10= -MLI*(B35*R55)*((I - B25T*MRI)*(B25*R55))
C
C     W1   - (NLL * NBR)      W1 = B35*R55
C     W2   - (NBR * NLR)      W2 = B25T*MRI
C     W3   - (NLR * NBR)      W3 = B25*R55
C     W4   - (NLC * NBC)      W4 = B14*S44
C     W5   - (NBC * NLR)      W5 = B24T*MRI
C     W6   - (NBC * NBC)      W6 = F3*W4 - I
C     W7   - (NBR * NBR)      W7 = W2*W3
C     W8   - (NBR * NBR)      W8 = I - W7
C     W9   - (NLL * NBC)      W9 = F4*B24
C     W10  - (NBC * NLL)      W10 = F7*B35T
C     W11  - (NLL * NBC)      W11 = W9 - B34
C     W12  - (NBC * NLL)      W12 = W10 - B34T
C     W13  - (NLL * NBE)      W13 = F4*B27
C     W14  - (NBC * NLJ)      W14 = F7*B85T - B84T
C     W15  - (NBC * NBE)      W15 = F3*B17
C     W16  - (NLL * NLJ)      W16 = -(F2 + M36) * B86T
C
C
C     R1   - (NLL,NBY)        R1  = F4 * B24  -  B34
C     R2   - (NBC,NJX)        R2  = F7*BX5T  -  BX4T
C
C
C     G11  - (NLL * NBE)  SUBMATRIX OF THE PARTITIONED MATRIX  G
C     G12  - (NLL * NLJ)  SUBMATRIX OF THE PARTITIONED MATRIX  G
C     G21  - (NBC * NBE)  SUBMATRIX OF THE PARTITIONED MATRIX  G
C     G22  - (NBC * NLJ)  SUBMATRIX OF THE PARTITIONED MATRIX  G
C
C     Q11  - (NLL * NBE)                IDENTICALLY ZERO
C     Q12  - (NLL * NLJ)  SUBMATRIX OF THE PARTITIONED MATRIX  Q
C     Q21  - (NBC * NBE)  SUBMATRIX OF THE PARTITIONED MATRIX  Q
C     Q22  - (NBC * NLJ)                IDENTICALLY ZERO       Q
C
C
C     H11  - (NLL,NBY)    SUBMATRIX OF THE PARTITIONED MATRIX  H
C     H12  - (NLL,NJX)    SUBMATRIX OF THE PARTITIONED MATRIX  H
C     H21  - (NBC,NBY)    SUBMATRIX OF THE PARTITIONED MATRIX  H
C     H22  - (NBL,NJX)    SUBMATRIX OF THE PARTITIONED MATRIX  H
 
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
 
      common /tapes/  nouttp
      COMMON /CNTRLS/ CTRLS(160)
      COMMON /CLASNO/ NBY,NBE,NBC,NBR,NBL,NLC,NLRR,NLL,NJXX,
     *                NJ8DC,NJ0,NJ9,NLJ,N,NSIND,NSDEP,JSKP6(6),NLR,NJX
      COMMON /ICODEZ/ IC11Z,IR22Z,IL33Z,IC44Z,IR55Z,IL66Z,IM33Z,IM36Z,
     *                IM66Z,IB14Z,IB24Z,IB25Z,IB34Z,IB35Z,IB36Z,IB17Z,
     *                IB27Z,IB37Z,IB84Z,IB85Z,IB86Z,IB87Z,IBX4Z,IBX5Z,
     *                IB2YZ,IB3YZ,IMSIZ,IMLIZ,IMRIZ
      COMMON /NCODEZ/ IF2Z, IF3Z, IF4Z, IF5Z, IF6Z, IF7Z, IF8Z ,IF9Z,
     .                IF10Z,IW1Z, IW2Z, IW3Z, IW4Z, IW5Z, IW6Z, IW7Z,
     .                IW9Z, IW10Z,IW11Z,IW12Z,IW13Z,IW14Z,IW15Z,IW16Z
C-----------------------------------------------------------------------
C      DIMENSION     B14(NLC,NBC),                          B17(NLC,NBE)
C     C              B24(NLR,NBC),B25(NLR,NBR),             B27(NLR,NBE)
C     C              B34(NLL,NBC),B35(NLL,NBR),B36(NLL,NBL),B37(NLL,NBE)
C     C              B84(NLJ,NBC),B85(NLJ,NBR),B86(NLJ,NBL),B87(NLJ,NBE)
C      DIMENSION BX5(NJX,NBR),BX4(NJX,NBC)
C      DIMENSION B3Y(NLL,NBY),B2Y(NLR,NBY)
C      DIMENSION     MSI(NLC,NLC),MRI(NLR,NLR),MLI(NLL,NLL),L66(NBL,NBL)
C     C              M36(NLL,NBL)
C      DIMENSION     F1 (NLL,NLL),F2 (NLL,NBL),F3 (NBC,NLC),F4 (NLL,NLR)
C     C              F5 (NLL,NLL),F6 (NBC,NLR),F7 (NBC,NBR),F8 (NBC,NBC)
C     C              F9 (NBC,NLR),F10(NLL,NBR)
C      DIMENSION     W1 (NLL,NBR),W2 (NBR,NLR),W3 (NLR,NBR),W4 (NLC,NBC)
C     C              W5 (NBC,NLR),W6 (NBC,NBC),W7 (NBR,NBR),W8 (NBR,NBR)
C     C              W9 (NLL,NBC),W10(NBC,NLL),W11(NLL,NBC),W12(NBC,NLL)
C     C              W13(NLL,NBE),W14(NBC,NLJ),W15(NBC,NBE),W16(NLL,NLJ)
CNOTE    W14 IS NOW NBC*NLJ ********
C      DIMENSION R1(NLL,NBY),R2(NBC,NJX)
C      DIMENSION A(N,1),G(N,1),Q(N,1),H(N,1)
C
C      DIMENSION    C44(NBC),R55(NBR),S44(NBC)
C-----------------------------------------------------------------------
      DIMENSION     B14(  1,  1),                          B17(  1,  1),
     C              B24(  1,  1),B25(  1,  1),             B27(  1,  1),
     C              B34(  1,  1),B35(  1,  1),B36(  1,  1),B37(  1,  1),
     C              B84(  1,  1),B85(  1,  1),B86(  1,  1),B87(  1,  1),
     *              BX5(  1,  1),BX4(  1,  1),
     *              B3Y(  1,  1),B2Y(  1,  1),
     *              MSI(  1,  1),MRI(  1,  1),MLI(  1,  1),L66(  1,  1),
     C              M36(  1,  1),
     *              F1 (  1,  1),F2 (  1,  1),F3 (  1,  1),F4 (  1,  1),
     C              F5 (  1,  1),F6 (  1,  1),F7 (  1,  1),F8 (  1,  1),
     C              F9 (  1,  1),F10(  1,  1),
     *              W1 (  1,  1),W2 (  1,  1),W3 (  1,  1),W4 (  1,  1),
     C              W5 (  1,  1),W6 (  1,  1),W7 (  1,  1),W8 (  1,  1),
     C              W9 (  1,  1),W10(  1,  1),W11(  1,  1),W12(  1,  1),
     C              W13(  1,  1),W14(  1,  1),W15(  1,  1),W16(  1,  1),
     *              R1 (  1,  1),R2 (  1,  1),
     *              A  (  N,  1),G  (  N,  1),Q  (  N,  1),H  (  N,  1)
 
      DIMENSION    C44(1),R55(1),S44(1)
 
      EQUIVALENCE  (NLC,NRC11,NRB14,NRMSI,NRB17,NCC11,NCMSI,NCF3 )
      EQUIVALENCE  (NLR,NRR22,NRB24,NRB25,NRMRI,NRB27,NCR22,NCMRI,
     C                  NCF4 ,NCF6 ,NCF9 )
      EQUIVALENCE  (NLL,NRL33,NRM33,NRM36,NRB34,NRB35,NRB36,NRMLI,
     C                  NRB37,NRF1 ,NRF2 ,NRF4 ,NRF5 ,NRF10,
     *                  NCL33,NCM33,NCMLI,NCF1)
      EQUIVALENCE (NLJ,NLJAC)
      EQUIVALENCE  (NLJ,NRB84,NRB85,NRB86,NRB87)
      EQUIVALENCE  (NBC,NRC44,NRF3 ,NRF6 ,NRF7 ,NRF8 ,NRF9 ,NCC44,
     *                  NCB14,NCB24,NCB34,NCB84,NCF8,NRS44,NCS44)
      EQUIVALENCE  (NBR,NRR55,NCR55,NCB25,NCB35,NCB85,NCF5 ,NCF7 ,NCF10)
      EQUIVALENCE  (NBL,NRL66,NRM66,NCL66,NCM36,NCM66,NCB36,NCB86,NCF2 )
      EQUIVALENCE  (NBE,NCB17,NCB27,NCB37,NCB87)
      EQUIVALENCE  (NLC,NRW4 ),
     C             (NLR,NRW3 ,NCW2 ,NCW5 ),
     C             (NLL,NRW1 , NRW9 ,NRW11,NCW10,NCW12),
     C             (NBC,NRW5 ,NRW6 ,NRW10,NRW12,NCW4 ,NCW6 ,NCW9,NCW11),
     C             (NBR,NRW2 ,NRW7 ,NRW8 ,NCW1 ,NCW3 ,NCW7 ,NCW8)
 
      EQUIVALENCE  (IC44Z,IS44Z)
      EQUIVALENCE  (IC11Z,IS11Z)
 
      EQUIVALENCE (IF1Z,IMLIZ),(IW7Z,IW8Z)
 
C * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C
C             CALCULATE A - MATRIX
C
C * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

C     ZERO OUT A MATRIX
 
      CALL YY01(A,N)
 
CSTEP 1       EQUIVALENCE    F1 = MLI
 
      IF (NLL.EQ. 0) GO TO 2062
      CALL YY02(F1,MLI,NLL)
 2062 CONTINUE
 
CSTEP 2       CALCULATE      W1 = B35 * R55
 
      CALL LIBMUX(W1 ,IW1Z ,NRW1 ,NCW1 ,B35,IB35Z, 0, 0,NRB35,NCB35,
     C                                  R55,IR55Z, 0, 1,NRR55,NCR55)
 
CSTEP 3       CALCULATE      W2 = B25T * MRI
 
      CALL LIBMUX(W2 ,IW2Z ,NRW2 ,NCW2 ,B25,IB25Z, 1, 0,NRB25,NCB25,
     C                                  MRI,IMRIZ, 0, 0,NRMRI,NCMRI)
 
CSTEP 4       CALCULATE      F4 = W1 * W2
C                            F4 = (B35*R55)*(B25T*MRI)
      CALL MXMULT(F4 ,IF4Z ,NRF4 ,NCF4 ,W1 ,IW1Z , 0, 0,NRW1 ,NCW1 ,
     C                                  W2 ,IW2Z , 0, 0,NRW2 ,NCW2 )
 
CSTEP 5       CALCULATE      W3 = B25 * R55
 
      CALL LIBMUX(W3 ,IW3Z ,NRW3 ,NCW3 ,B25,IB25Z, 0, 0,NRB25,NCB25,
     C                                  R55,IR55Z, 0, 1,NRR55,NCR55)
 
CSTEP 6       CALCULATE      W7 = W2 * W3
C                            W7 = (B25T*MRI)*(B25*R55)
      CALL MXMULT(W7 ,IW7Z ,NRW7 ,NCW7 ,W2 ,IW2Z , 0, 0,NRW2 ,NCW2 ,
     C                                  W3 ,IW3Z , 0, 0,NRW3 ,NCW3 )
 
CSTEP 7       CALCULATE      W8 = I - W7
 
      IW8Z=0
      IF(NBR.LE.0)GO TO 50
      IW8Z=1
      CALL YY03(W8,W7,NBR,NRW7,NCW7)
 
CNOTE         WE ARE FINISHED WITH W2 AND W7 STORAGE AREAS
CSTEP 8       CALCULATE      F5 = W1 * W8
 
   50 CONTINUE
      CALL MXMULT(F5 ,IF5Z ,NRF5 ,NCF5 ,W1 ,IW1Z , 0, 0,NRW1 ,NCW1 ,
     C                                  W8 ,IW8Z , 0, 0,NRW8 ,NCW8 )
 
CNOTE         WE ARE FINISHED WITH W1 AND W8 STORAGE AREAS
CSTEP 9       CALCULATE      F3 = B14T * MSI
 
      CALL LIBMUX(F3 ,IF3Z ,NRF3 ,NCF3 ,B14,IB14Z, 1, 0,NRB14,NCB14,
     C                                  MSI,IMSIZ, 0, 0,NRMSI,NCMSI)
 
CSTEP 10      CALCULATE      W4 = B14 * S44
 
      IF(NBC.LE.0)GO TO 51
      DO 10 I = 1,NRC44
   10 S44(I)=1.0D0/C44(I)
      CALL LIBMUX(W4 ,IW4Z ,NRW4 ,NCW4 ,B14,IB14Z, 0, 0,NRB14,NCB14,
     C                                  S44,IS44Z, 0, 1,NRS44,NCS44)
   51 CONTINUE
 
CSTEP 11      CALCULATE      W5 = B24T * MRI
 
      CALL LIBMUX(W5 ,IW5Z ,NRW5 ,NCW5 ,B24,IB24Z, 1, 0,NRB24,NCB24,
     C                                  MRI,IMRIZ, 0, 0,NRMRI,NCMRI)
 
CSTEP 12      CALCULATE      W6 = F3 * W4 - I
 
      IW6Z=0
      IF(NRW6.LE.0)GO TO 52
      CALL MXMULT(W6 ,IW6Z ,NRW6 ,NCW6 ,F3 ,IF3Z , 0, 0,NRF3 ,NCF3 ,
     C                                  W4 ,IW4Z , 0, 0,NRW4 ,NCW4 )
      IW6Z = 1
      CALL YY04(W6,NBC,NRW6)
   52 CONTINUE
 
CSTEP 13      CALCULATE      F6 = W6 * W5
 
      CALL MXMULT(F6 ,IF6Z ,NRF6 ,NCF6 ,W6 ,IW6Z , 0, 0,NRW6 ,NCW6 ,
     C                                  W5 ,IW5Z , 0, 0,NRW5 ,NCW5 )
 
CSTEP 14      CALCULATE      F7 = W5 * W3
 
      CALL MXMULT(F7 ,IF7Z ,NRF7 ,NCF7 ,W5 ,IW5Z , 0, 0,NRW5 ,NCW5 ,
     C                                  W3 ,IW3Z , 0, 0,NRW3 ,NCW3 )
 
CSTEP 15      CALCULATE      F8 = S44 * W6
 
      CALL MXMULT(F8 ,IF8Z ,NRF8 ,NCF8 ,S44,IS44Z, 0, 1,NRS44,NCS44,
     C                                  W6 ,IW6Z , 0, 0,NRW6 ,NCW6 )
 
CSTEP 16      CALCULATE      F9 = S44 * F6
 
      CALL MXMULT(F9 ,IF9Z ,NRF9 ,NCF9 ,S44,IS44Z, 0, 1,NRS44,NCS44,
     C                                  F6 ,IF6Z , 0, 0,NRF6 ,NCF6 )
 
CSTEP 17      CALCULATE      F10 = (-1) * (F1 * F5)
 
      IF10Z=0
      IF (IF1Z.EQ.0 .OR. IF5Z.EQ.0)  GO TO 53
      IF(NRF10.LE.0.OR.NCF10.LE.0.) GO TO 53
      CALL MXMULT(F10,IF10Z,NRF10,NCF10,F1 ,IF1Z , 0, 0,NRF1 ,NCF1 ,
     C                                  F5 ,IF5Z , 0, 0,NRF5 ,NCF5 )
      CALL YY05(F10,NLL,NBR,NRF10,NCF10)
   53 CONTINUE
 
CSTEP 18      CALCULATE      W9 = F4 * B24
 
      CALL RIBMUX(W9 ,IW9Z ,NRW9 ,NCW9 ,F4 ,IF4Z , 0, 0,NRF4 ,NCF4 ,
     C                                  B24,IB24Z, 0, 0,NRB24,NCB24)
 
CSTEP 19      CALCULATE      W10 = F7 * B35T
 
      CALL RIBMUX(W10,IW10Z,NRW10,NCW10,F7 ,IF7Z , 0, 0,NRF7 ,NCF7 ,
     C                                  B35,IB35Z, 1, 0,NRB35,NCB35)
 
CSTEP 20      CALCULATE      W11 = W9 - B34
 
      IW11Z=0
      IF (IB34Z.EQ.0 .AND. IW9Z.EQ.0)  GO TO 54
      IW11Z=1
      CALL YY06 (W11,W9,B34,NLL,NBC,NRW11,NCW11)
   54 CONTINUE
 
CSTEP 21      CALCULATE      W12 = W10 - B34T
CNOTE          B34T IS HANDLED BY CORRECTLY INDEXING B34
 
      IW12Z=0
      IF (IW10Z.EQ.0 .AND. IB34Z.EQ.0)  GO TO 55
      IW12Z=1
      CALL YY07(W12,W10,B34,NLL,NBC,NRW12,NCW12)
   55 CONTINUE
 
CSTEP 22      CALCULATE      A11 = F10 * B35T
 
      IF (IF10Z.EQ.0 .OR. IB35Z.EQ.0)  GO TO 8845
      CALL YY08(A,N,F10,B35,NLL,NBR)
 
CSTEP 23      CALCULATE      A12 = F1 * W11
 
 8845 IF (IF1Z.EQ.0 .OR. IW11Z.EQ.0)  GO TO 8283
      CALL YY09(A,N,F1,W11,NLL,NBC)
 
CSTEP 24      CALCULATE      A21 = F8 * W12
 
 8283 IF (IF8Z.EQ.0 .OR. IW12Z.EQ.0)  GO TO 7723
      CALL YY10(A,N,F8,W12,NBC,NLL)
 
CSTEP 25      CALCULATE      A22 = F9 * B24
 
 7723 IF (IF9Z.EQ.0 .OR. IB24Z.EQ.0)  GO TO 5657
      CALL YY11(A,N,F9,B24,NBC,NLR,NLL)
 5657 CONTINUE
 
CSTEP 36         PRINT THE  A - MATRIX
 
      IF (nint(CTRLS(76)).gt.0) then
         write (nouttp,16)
  16     FORMAT ('0AGNEW2: A  MATRIX')
         CALL OURMAT ( A ,N,N,N,N)
      end if
 
C * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C
C             CALCULATE G - MATRIX
C
C * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

C     ZERO OUT G MATRIX
 
      CALL YY12(G,N,NSIND)
 
CSTEP 26     CALCULATE  G11 = -F1*(B37 - F4*B27)
 
      IF (NLL.EQ.0 .OR. NBE.EQ.0)  GO TO 60
 
C     W13 = F4*B27 - B37
 
      CALL RIBMUX (W13,IW13Z,NLL,NBE,F4,IF4Z,0,0,NLL,NLR,
     *               B27,IB27Z,0,0,NLR,NBE)
      IF (IB37Z.EQ.0)  GO TO 332
      IW13Z = 1
      CALL YY13(W13,B37,NLL,NBE)
 
CPART 2     G11 = F1*W13
 
  332 IF (IW13Z.EQ.0 .OR. IF1Z.EQ.0)  GO TO 60
      CALL YY14(G,N,F1,W13,NLL,NBE)
 
CSTEP 27     G12 = F10 * B85T
 
   60 IF (IF10Z.EQ.0 .OR. IB85Z.EQ.0) GO TO 346
      CALL YY15(G,N,F10,B85,NLL,NBR,NLJ,NBE)
 
CSTEP 28     G21 = F9 * B27
 
  346 IF (IF9Z.EQ.0 .OR. IB27Z.EQ.0) GO TO 482
      CALL YY16(G,N,F9,B27,NBC,NLR,NBE,NLL)
 
CSTEP 29     G22 = F8 * (F7*B85T-B84T)
CPART 1      W14 = F7*B85T -B84T
CNOTE **** W14 IS NOW DIMENSIONED NBC * NLJ
 
  482 IF (NBC.LE.0 .OR. NLJ.LE.0) GO TO 631
      IF (IB84Z.EQ.0) GO TO 607
      IW14Z = 1
      CALL YY17(W14,B84,NBC,NLJ)
  607 IF (NBR.LE.0) GO TO 609
      CALL RIBMUX (W14, IW14Z, NBC, NLJ, F7, IF7Z, 0, 0, NBC, NBR,
     *               B85, IB85Z, 1, 0, NLJ, NBR)
  609 IF (IW14Z.EQ.0 .OR. IF8Z.EQ.0) GO TO 631
      CALL YY18(G,N,F8,W14,NBC,NLJ,NLL,NBE)

CSTEP 45      PRINT  G MATRIX
 
  631 IF (nint(CTRLS(76)).gt.0) then
         WRITE (nouttp,7746)
 7746    FORMAT ('0AGNEW2: G  MATRIX')
         CALL OURMAT (G,N,NSIND,N,NSIND)
      end if 
 
      CALL       GANEW3 (C44,R55,S44,B14,B17,B24,B25,B27,B34,B35,B36,
     C           B37,B84,B85,B86,B87,MSI,MRI,MLI,L66,M36,F1,F2,F3,F4,
     C           F5,F6,F7,F8,F9,F10,W1,W2,W3,W4,W5,W6,W7,W8,W9,W10,W11,
     C           W12,W13,W14,W15,W16,R1,R2,A,G,Q,H,
     C           BX5,BX4,B3Y,B2Y)
      RETURN
 
      END
 
      SUBROUTINE YY01(A,N)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION A(N,1)
      DO 104 I=1,N
      DO 104 J=1,N
  104 A(I,J)=0.D0
      RETURN
      END
 
      SUBROUTINE YY02(F1,MLI,NLL)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION F1(NLL,NLL),MLI(NLL,NLL)
      DO 2061 I=1,NLL
      DO 2061 J=1,NLL
 2061 F1(I,J)=MLI(I,J)
      RETURN
      END
 
      SUBROUTINE YY03(W8,W7,NBR,NRW7,NCW7)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION W8(NBR,NBR),W7(NBR,NBR)
      DO 8 I = 1,NRW7
      DO 8 J = 1,NCW7
      IF(I.EQ.J) GO TO 18
      W8(I,J) = -W7(I,J)
      GO TO 8
   18 W8(I,I) = 1.0D0 - W7(I,I)
    8 CONTINUE
      RETURN
      END
 
      SUBROUTINE YY04(W6,NBC,NRW6)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION W6(NBC,NBC)
      DO 12 I = 1,NRW6
      W6(I,I) = W6(I,I) - 1.0D0
   12 CONTINUE
      RETURN
      END
 
      SUBROUTINE YY05(F10,NLL,NBR,NRF10,NCF10)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION F10(NLL,NBR)
      DO 17 I = 1,NRF10
      DO 17 J = 1,NCF10
      F10(I,J) = -F10(I,J)
   17 CONTINUE
      RETURN
      END
 
      SUBROUTINE  YY06 (W11,W9,B34,NLL,NBC,NRW11,NCW11)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION W11(NLL,NBC),W9(NLL,NBC),B34(NLL,NBC)
      DO 20 I = 1,NRW11
      DO 20 J = 1,NCW11
      W11(I,J) = W9(I,J) - B34(I,J)
   20 CONTINUE
      RETURN
      END
 
      SUBROUTINE YY07(W12,W10,B34,NLL,NBC,NRW12,NCW12)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION W12(NBC,NLL),W10(NBC,NLL),B34(NLL,NBC)
      DO 21 I = 1,NRW12
      DO 21 J = 1,NCW12
      W12(I,J) = W10(I,J) - B34(J,I)
   21 CONTINUE
      RETURN
      END
 
      SUBROUTINE YY08(A,N,F10,B35,NLL,NBR)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION A(N,1),F10(NLL,NBR),B35(NLL,NBR)
      DO 8844 I=1,NLL
      DO 8844 J=1,NLL
      SUM=0.d0
      DO 8843 K=1,NBR
 8843 SUM=SUM + F10(I,K)*B35(J,K)
 8844 A(I,J)=SUM
      RETURN
      END
 
      SUBROUTINE YY09(A,N,F1,W11,NLL,NBC)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION A(N,1),F1(NLL,NLL),W11(NLL,NBC)
      DO 8282 I=1,NLL
      DO 8282 JJ=1,NBC
      J=NLL+JJ
      SUM=0.d0
      DO 8281 K=1,NLL
 8281 SUM=SUM + F1(I,K)*W11(K,JJ)
 8282 A(I,J)=SUM
      RETURN
      END
 
      SUBROUTINE YY10(A,N,F8,W12,NBC,NLL)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION A(N,1),F8(NBC,NBC),W12(NBC,NLL)
      DO 7722 II=1,NBC
      I=NLL+II
      DO 7722 J=1,NLL
      SUM=0.d0
      DO 7721 K=1,NBC
 7721 SUM=SUM + F8(II,K) * W12(K,J)
 7722 A(I,J)=SUM
      RETURN
      END
 
      SUBROUTINE YY11(A,N,F9,B24,NBC,NLR,NLL)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION A(N,1),F9(NBC,NLR),B24(NLR,NBC)
      DO 5856 II=1,NBC
      I=NLL+II
      DO 5856 JJ=1,NBC
      J=NLL+JJ
      SUM=0.d0
      DO 5855 K=1,NLR
 5855 SUM=SUM + F9(II,K)*B24(K,JJ)
 5856 A(I,J)=SUM
      RETURN
      END
 
      SUBROUTINE YY12(G,N,NSIND)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION G(N,1)
      DO 328 I=1,N
      DO 328 J=1,NSIND
  328 G(I,J)=0.D0
      RETURN
      END
 
      SUBROUTINE YY13(W13,B37,NLL,NBE)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION W13(NLL,NBE),B37(NLL,NBE)
      DO 331 I=1, NLL
      DO 331 J=1, NBE
  331 W13 (I,J) = W13 (I,J) - B37 (I,J)
      RETURN
      END
 
      SUBROUTINE YY14(G,N,F1,W13,NLL,NBE)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION G(N,1),F1(NLL,NLL),W13(NLL,NBE)
      DO 337 I = 1, NLL
      DO 337 J = 1, NBE
      SUM = 0.d0
      DO 336 K = 1, NLL
  336 SUM = SUM + F1(I,K) * W13(K,J)
  337 G(I,J) = SUM
      RETURN
      END
 
      SUBROUTINE YY15(G,N,F10,B85,NLL,NBR,NLJ,NBE)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION G(N,1),F10(NLL,NBR),B85(NLJ,NBR)
      DO 345 I = 1, NLL
      DO 345 JJ = 1, NLJ
      J = NBE + JJ
      SUM = 0.d0
      DO 344 K = 1, NBR
  344 SUM = SUM + F10(I,K) * B85(JJ,K)
  345 G(I,J) = SUM
      RETURN
      END
 
      SUBROUTINE YY16(G,N,F9,B27,NBC,NLR,NBE,NLL)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION G(N,1),F9(NBC,NLR),B27(NLR,NBE)
      DO 481 II = 1, NBC
      I = NLL + II
      DO 481 J = 1, NBE
      SUM = 0.d0
      DO 480 K = 1, NLR
  480 SUM = SUM + F9(II,K) * B27(K,J)
  481 G(I,J) = SUM
      RETURN
      END
 
      SUBROUTINE YY17(W14,B84,NBC,NLJ)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION W14(NBC,NLJ),B84(NLJ,NBC)
      DO 606 I = 1, NBC
      DO 606 J = 1, NLJ
  606 W14(I,J) = - B84(J,I)
      RETURN
      END
 
      SUBROUTINE YY18(G,N,F8,W14,NBC,NLJ,NLL,NBE)
      IMPLICIT DOUBLE PRECISION (A,C-H,L,M,O-Z),INTEGER*2(B)
      DIMENSION G(N,1),F8(NBC,NBC),W14(NBC,NLJ)
      DO 617 II = 1, NBC
      I = NLL + II
      DO 617 JJ = 1, NLJ
      J = NBE + JJ
      SUM = 0.d0
      DO 616 K = 1, NBC
  616 SUM = SUM + F8(II,K) * W14(K,JJ)
  617 G(I,J) = SUM
      RETURN
      END
