CSECT   CMPV1
C     REVISED 4/25/73
 
      SUBROUTINE CMPV1 (N,NM,M,IVEC,A,VECR, H,LAMDA, INDIC,
     *                  IWORK,SUBDIA,WORK1,WORK2,WORK,EPS,EX)
 
C THIS SUBROUTINE FINDS THE COMPLEX EIGENVECTOR OF THE REAL
C UPPER-HESSENBERG MATRIX OF ORDER N CORRESPONDING TO THE
C COMPLEX EIGENVALUE WITH THE REAL PART IN EVR(IVEC) AND THE
C CORRESPONDING IMAGINARY PART IN EVI(IVEC). THE INVERSE
C ITERATION METHOD IS USED MODIFIED TO AVOID THE USE OF
C COMPLEX ARITHMETIC.
C THE MATRIX ON WHICH THE INVERSE ITERATION IS PERFORMED IS
C BUILT UP IN THE ARRAY A BY USING THE UPPER-HESSENBERG
C MATRIX PRESERVED IN THE UPPER HALF OF THE ARRAY H AND IN
C THE ARRAY SUBDIA.
C NM DEFINES THE FIRST DIMENSION OF THE TWO DIMENSIONAL
C ARRAYS A, VECR AND H. NM MUST BE EQUAL TO OR GREATER
C THAN N.
C M IS THE ORDER OF THE SUBMATRIX OBTAINED BY A SUITABLE
C DECOMPOSITION OF THE UPPER-HESSENBERG MATRIX IF SOME
C SUBDIAGONAL ELEMENTS ARE EQUAL TO ZERO. THE VALUE OF M IS
C CHOSEN SO THAT THE LAST N-M COMPONENTS OF THE COMPLEX
C EIGENVECTOR ARE ZERO.
C
C THE REAL PARTS OF THE FIRST M COMPONENTS OF THE COMPUTED
C COMPLEX EIGENVECTOR WILL BE FOUND IN THE FIRST M PLACES OF
C THE COLUMN WHOSE TOP ELEMENT IS VECR(1,IVEC) AND THE
C CORRESPONDING IMAGINARY PARTS OF THE FIRST M COMPONENTS OF
C THE COMPLEX EIGENVECTOR WILL BE FOUND IN THE FIRST M
C PLACES OF THE COLUMN WHOSE TOP ELEMENT IS VECR(1,IVEC-1).
C
C THE ARRAY INDIC INDICATES THE SUCCESS OF THE ROUTINE AS
C FOLLOWS
C     VALUE OF INDIC(I)     EIGENVECTOR I
C            1                NOT FOUND
C            2                  FOUND
C THE ARRAYS IWORK, WORK1, WORK2 AND WORK ARE THE WORKING
C STORES USED DURING THE INVERSE ITERATION PROCESS.
C EPS IS A SMALL POSITIVE NUMBER THAT NUMERICALLY REPRESENTS
C ZERO IN THE PROGRAM. EPS = (EUCLIDIAN NORM OF H)*EX, WHERE
C EX = 2**(-T). T IS THE NUMBER OF BINARY DIGITS IN THE
C MANTISSA OF A FLOATING POINT NUMBER.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER (I-N)
C-------------------------------
C VAX only:
C     REAL*16 B,D,R,S,U,V,T
C-------------------------------
      DOUBLE PRECISION LAMDA (NM)
      DIMENSION A(NM,1),VECR(NM,1),H(NM,1),INDIC(NM),IWORK(NM),
     *          SUBDIA(NM),WORK1(NM),WORK2(NM),WORK(NM)
 
      FKSI=LAMDA(2*IVEC-1)
      ETA=LAMDA(2*IVEC)
 
C THE MODIFICATION OF THE EIGENVALUE (FKSI + I*ETA) IF MORE
C EIGENVALUES ARE EQUAL.
 
      IF(IVEC.EQ.M) GO TO 2
      K = IVEC+1
      R = 0.0D0
      DO 1 I=K,M
      IF (FKSI .NE. LAMDA(2*I-1)) GO TO 1
      IF (ABS(ETA) .NE. ABS(LAMDA(2*I)))GO TO 1
      R = R+3.0D0
    1 CONTINUE
      R = R*EX
      FKSI = FKSI+R
      ETA = ETA+R
 
C THE MATRIX ((H-FKSI*I)*(H-FKSI*I) + (ETA*ETA)*I) IS
C STORED INTO THE ARRAY A.
 
    2 R = FKSI*FKSI+ETA*ETA
      S = 2.0D0*FKSI
      L = M-1
      DO 5 I=1,M
      DO 4 J=I,M
      D = 0.0D0
      A(J,I) = 0.0D0
      DO 3 K=I,J
    3 D = D+H(I,K)*H(K,J)
    4 A(I,J) = D-S*H(I,J)
    5 A(I,I) = A(I,I)+R
      DO 9 I=1,L
      R = SUBDIA(I)
      A(I+1,I) = -S*R
      I1 = I+1
      DO 6 J=1,I1
    6 A(J,I) = A(J,I)+R*H(J,I+1)
      IF(I.EQ.1) GO TO 7
      A(I+1,I-1) = R*SUBDIA(I-1)
    7 DO 8 J=I,M
    8 A(I+1,J) = A(I+1,J)+R*H(I,J)
    9 CONTINUE
 
C THE GAUSSIAN ELIMINATION OF THE MATRIX
C ((H-FKSI*I)*(H-FKSI*I) + (ETA*ETA)*I) IN THE ARRAY A. THE
C ROW INTERCHANGES THAT OCCUR ARE INDICATED IN THE ARRAY
C IWORK. ALL THE MULTIPLIERS ARE STORED IN THE FIRST AND IN
C THE SECOND SUBDIAGONAL OF THE ARRAY A.
 
      K = M-1
      DO 18 I=1,K
      I1 = I+1
      I2 = I+2
      IWORK(I) = 0
      IF(I.EQ.K) GO TO 10
      IF(A(I+2,I).NE.0.0D0) GO TO 11
   10 IF(A(I+1,I).NE.0.0D0) GO TO 11
      IF(A(I,I).NE.0.0D0) GO TO 18
      A(I,I) = EPS
      GO TO 18
 
   11 IF(I.EQ.K) GO TO 12
      IF(ABS(A(I+1,I)).GE.ABS(A(I+2,I))) GO TO 12
      IF(ABS(A(I,I)).GE.ABS(A(I+2,I))) GO TO 16
      L = I+2
      IWORK(I) = 2
      GO TO 13
   12 IF(ABS(A(I,I)).GE.ABS(A(I+1,I))) GO TO 15
      L = I+1
      IWORK(I) = 1
 
   13 DO 14 J=I,M
      R = A(I,J)
      A(I,J) = A(L,J)
   14 A(L,J) = R
   15 IF(I.NE.K) GO TO 16
      I2 = I1
   16 DO 17 L=I1,I2
      R = -A(L,I)/A(I,I)
      A(L,I) = R
      DO 17 J=I1,M
   17 A(L,J) = A(L,J)+R*A(I,J)
   18 CONTINUE
      IF(A(M,M).NE.0.0D0) GO TO 19
      A(M,M) = EPS
 
C THE VECTOR (1,1,...,1) IS STORED INTO THE RIGHT-HAND SIDE
C VECTORS VECR( ,IVEC) AND VECR( ,IVEC-1) REPRESENTING THE
C COMPLEX RIGHT-HAND SIDE VECTOR.
 
   19 DO 21 I=1,N
      IF(I.GT.M) GO TO 20
      VECR(I,IVEC) = 1.0D0
      VECR(I,IVEC-1) = 1.0D0
      GO TO 21
   20 VECR(I,IVEC) = 0.0D0
      VECR(I,IVEC-1) = 0.0D0
   21 CONTINUE
 
C THE INVERSE ITERATION IS PERFORMED ON THE MATRIX UNTIL THE
C INFINITE NORM OF THE RIGHT-HAND SIDE VECTOR IS GREATER
C THAN THE BOUND DEFINED AS 0.01D0/(N*EX)
 
      BOUND = 0.01D0/(EX*dfloat(N))
      NS = 0
      ITER = 1
      DO 22 I=1,M
   22 WORK(I) = H(I,I)-FKSI
 
C THE SEQUENCE OF THE COMPLEX VECTORS Z(S) = P(S)+I*Q(S) AND
C W(S+1) = U(S+1)+I*V(S+1) IS GIVEN BY THE RELATIONS
C (A - (FKSI-I*ETA)*I)*W(S+1) = Z(S) AND
C Z(S+1) = W(S+1)/MAX(W(S+1)).
C THE FINAL W(S) IS TAKEN AS THE COMPUTED EIGENVECTOR.
C
C THE COMPUTATION OF THE RIGHT-HAND SIDE VECTOR
C (A-FKSI*I)*P(S)-ETA*Q(S). A IS AN UPPER-HESSENBERG MATRIX.
 
   23 DO 27 I=1,M
      D = WORK(I)*VECR(I,IVEC)
      IF(I.EQ.1) GO TO 24
      D = D+SUBDIA(I-1)*VECR(I-1,IVEC)
   24 L = I+1
      IF(L.GT.M) GO TO 26
      DO 25 K=L,M
   25 D = D+H(I,K)*VECR(K,IVEC)
   26 VECR(I,IVEC-1) = D-ETA*VECR(I,IVEC-1)
   27 CONTINUE
 
C GAUSSIAN ELIMINATION OF THE RIGHT-HAND SIDE VECTOR.
 
      K = M-1
      DO 28 I=1,K
      L = I+IWORK(I)
      R = VECR(L,IVEC-1)
      VECR(L,IVEC-1) = VECR(I,IVEC-1)
      VECR(I,IVEC-1) = R
      VECR(I+1,IVEC-1) = VECR(I+1,IVEC-1)+A(I+1,I)*R
      IF(I.EQ.K) GO TO 28
      VECR(I+2,IVEC-1) = VECR(I+2,IVEC-1)+A(I+2,I)*R
   28 CONTINUE
 
C THE COMPUTATION OF THE REAL PART U(S+1) OF THE COMPLEX
C VECTOR W(S+1). THE VECTOR U(S+1) IS OBTAINED AFTER THE
C BACKSUBSTITUTION.
 
      DO 31 I=1,M
      J = M-I+1
      D = VECR(J,IVEC-1)
      IF(J.EQ.M) GO TO 30
      L = J+1
      DO 29 K = L,M
      D1 = A(J,K)
   29 D = D-D1*VECR(K,IVEC-1)
   30 VECR(J,IVEC-1) = D/A(J,J)
   31 CONTINUE
 
C THE COMPUTATION OF THE IMAGINARY PART V(S+1) OF THE VECTOR
C W(S+1), WHERE V(S+1) = (P(S)-(A-FKSI*I)*U(S+1))/ETA.
 
      DO 35 I=1,M
      D = WORK(I)*VECR(I,IVEC-1)
      IF(I.EQ.1) GO TO 32
      D = D+SUBDIA(I-1)*VECR(I-1,IVEC-1)
   32 L = I+1
      IF(L.GT.M) GO TO 34
      DO 33 K=L,M
   33 D = D+H(I,K)*VECR(K,IVEC-1)
   34 VECR(I,IVEC) = (VECR(I,IVEC)-D)/ETA
   35 CONTINUE
 
C THE COMPUTATION OF (INFIN. NORM OF W(S+1))**2.
 
      L = 1
      S = 0.0D0
      DO 36 I=1,M
C IBM:
      R = VECR(I,IVEC)**2+VECR(I,IVEC-1)**2
c-------------------------------------------------
C VAX:
C      R=VECR(I,IVEC)
C      R=R*R
C      T=VECR(I,IVEC-1)
C      T=T*T
C      R=R+T
c---------------------------------------------------
      IF(R.LE.S) GO TO 36
      S = R
      L = I
   36 CONTINUE
 
C THE COMPUTATION OF THE VECTOR Z(S+1), WHERE Z(S+1) = W(S+1)/
C (COMPONENT OF W(S+1) WITH THE LARGEST ABSOLUTE VALUE).
 
      U = VECR(L,IVEC-1)
      V = VECR(L,IVEC)
      DO 37 I=1,M
      B = VECR(I,IVEC)
      R = VECR(I,IVEC-1)
      VECR(I,IVEC) = (R*U+B*V)/S
   37 VECR(I,IVEC-1) = (B*U-R*V)/S
 
C THE COMPUTATION OF THE RESIDUALS AND COMPARISON OF THE
C RESIDUALS OF THE TWO SUCCESSIVE STEPS OF THE INVERSE
C ITERATION. IF THE INFINITE NORM OF THE RESIDUAL VECTOR IS
C GREATER THAN THE INFINITE NORM OF THE PREVIOUS RESIDUAL
C VECTOR THE COMPUTED VECTOR OF THE PREVIOUS STEP IS TAKEN
C AS THE COMPUTED APPROXIMATION TO THE EIGENVECTOR.
 
      B = 0.0D0
      DO 41 I=1,M
      R = WORK(I)*VECR(I,IVEC-1)-ETA*VECR(I,IVEC)
      U = WORK(I)*VECR(I,IVEC)+ETA*VECR(I,IVEC-1)
      IF(I.EQ.1) GO TO 38
      R = R+SUBDIA(I-1)*VECR(I-1,IVEC-1)
      U = U+SUBDIA(I-1)*VECR(I-1,IVEC)
   38 L = I+1
      IF(L.GT.M) GO TO 40
      DO 39 J=L,M
      R = R+H(I,J)*VECR(J,IVEC-1)
   39 U = U+H(I,J)*VECR(J,IVEC)
   40 U = R*R+U*U
      IF(B.GE.U) GO TO 41
      B = U
   41 CONTINUE
      IF(ITER.EQ.1) GO TO 42
      IF(PREVIS.LE.B) GO TO 44
   42 DO 43 I=1,N
      WORK1(I) = VECR(I,IVEC)
   43 WORK2(I) = VECR(I,IVEC-1)
      PREVIS = B
      IF(NS.EQ.1) GO TO 46
      IF(ITER.GT.6) GO TO 47
      ITER = ITER+1
      IF(BOUND.GT.SQRT(S)) GO TO 23
      NS = 1
      GO TO 23
 
   44 DO 45 I=1,N
      VECR(I,IVEC) = WORK1(I)
   45 VECR(I,IVEC-1) = WORK2(I)
   46 INDIC(IVEC-1) = 2
      INDIC(IVEC) = 2
   47 RETURN
 
      END
