CSECT   EIGEN1
C     REVISED 4/25/73
 
      SUBROUTINE EIGEN1(N,NM,A,T,LAMDA,VECR,VECI,INDIC,AC)
 
C THIS IS A DOUBLE PRECISION VERSION OF THE SUBROUTINE EIGENP DESIGNED
C TO RUN ON THE IBM SYSTEM/360.  ALL VARIABLES BEGINNING WITH THE
C LETTERS A THROUGH H AND O THROUGH Z ARE ASSUMED TO BE REAL*8
C VARIABLES.  ALL VARIABLES BEGINNING WITH THE LETTERS I THROUGH N
C ARE ASSUMED TO BE INTEGER*4 VARIABLES.
C
C THIS SUBROUTINE FINDS ALL THE EIGENVALUES AND THE
C EIGENVECTORS OF A REAL GENERAL MATRIX OF ORDER N.
C
C FIRST IN THE SUBROUTINE SCALE THE MATRIX IS SCALED SO THAT
C THE CORRESPONDING ROWS AND COLUMNS ARE APPROXIMATELY
C BALANCED AND THEN THE MATRIX IS NORMALISED SO THAT THE
C VALUE OF THE EUCLIDIAN NORM OF THE MATRIX IS EQUAL TO ONE.
C
C THE EIGENVALUES ARE COMPUTED BY THE QR OR DOUBLE-STEP METHOD
C IN THE SUBROUTINE HESQR.
C THE EIGENVECTORS ARE COMPUTED BY INVERSE ITERATION IN
C THE SUBROUTINE REALVE, FOR THE REAL EIGENVALUES, OR IN THE
C SUBROUTINE COMPVE, FOR THE COMPLEX EIGENVALUES.
C
C THE ELEMENTS OF THE MATRIX ARE TO BE STORED IN THE FIRST N
C ROWS AND COLUMNS OF THE TWO DIMENSIONAL ARRAY A. THE
C ORIGINAL MATRIX IS DESTROYED BY THE SUBROUTINE.
C N IS THE ORDER OF THE MATRIX.
C NM DEFINES THE FIRST DIMENSION OF THE TWO DIMENSIONAL
C ARRAYS A, VECR, VECI AND THE DIMENSION OF THE ONE
C DIMENSIONAL ARRAYS EVR, EVI AND INDIC. THEREFORE THE
C CALLING PROGRAM SHOULD CONTAIN THE FOLLOWING DECLARATION
C     DIMENSION A(NM,NN),VECR(NM,NN),VECI(NM,NN),
C    *          EVR(NM),EVI(NM),INDIC(NM)
C WHERE NM AND NN ARE ANY NUMBERS EQUAL TO OR GREATER THAN N
C THE UPPER LIMIT FOR NM IS EQUAL TO 100 BUT MAY BE
C INCREASED TO THE VALUE MAX BY REPLACING THE DIMENSION
C STATEMENT
C     DIMENSION IWORK(100),LOCAL(100), ... ,WORK(100)
C IN THE SUBROUTINE EIGENP WITH
C     DIMENSION IWORK(MAX),LOCAL(MAX), ... ,WORK(MAX)
C NM AND NN ARE OF COURSE BOUNDED BY THE SIZE OF THE STORE.
C
C THE REAL PARAMETER T MUST BE SET EQUAL TO THE NUMBER OF
C BINARY DIGITS IN THE MANTISSA OF A SINGLE PRECISION
C FLOATING-POINT NUMBER.
C
C THE REAL PARTS OF THE N COMPUTED EIGENVALUES WILL BE FOUND
C IN THE FIRST N PLACES OF THE ARRAY EVR AND THE IMAGINARY
C PARTS IN THE FIRST N PLACES OF THE ARRAY EVI.
C THE REAL COMPONENTS OF THE NORMALISED EIGENVECTOR I
C (I=1,2,...,N) CORRESPONDING TO THE EIGENVALUE STORED IN
C EVR(I) AND EVI(I) WILL BE FOUND IN THE FIRST N PLACES OF
C THE COLUMN I OF THE TWO DIMENSIONAL ARRAY VECR AND THE
C IMAGINARY COMPONENTS IN THE FIRST N PLACES OF THE COLUMN I
C OF THE TWO DIMENSIONAL ARRAY VECI.
C
C THE REAL EIGENVECTOR IS NORMALISED SO THAT THE SUM OF THE
C SQUARES OF THE COMPONENTS IS EQUAL TO ONE.
C THE COMPLEX EIGENVECTOR IS NORMALISED SO THAT THE
C COMPONENT WITH THE LARGEST VALUE IN MODULUS HAS ITS REAL
C PART EQUAL TO ONE AND THE IMAGINARY PART EQUAL TO ZERO.
C
C THE ARRAY INDIC INDICATES THE SUCCESS OF THE SUBROUTINE
C EIGENP AS FOLLOWS
C     VALUE OF INDIC(I)    EIGENVALUE I    EIGENVECTOR I
C            0               NOT FOUND       NOT FOUND
C            1               FOUND           NOT FOUND
C            2               FOUND           FOUND
 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER (I-N)
 
      COMPLEX*16      AC
      COMMON /CURNTS/ WORK(1)
      COMMON /DERIVS/ WORK1(1)
      COMMON /VOLTGS/ WORK2(1)
      COMMON /SAVEIC/ SUBDIA(1)
      COMMON /SAVICT/ PRFACT(1)
      DOUBLE PRECISION LAMDA (NM)
      DIMENSION       AC(NM,1)
      DIMENSION       A(NM,1),VECR(NM,1),VECI(NM,1),INDIC(NM)
      DIMENSION       IWORK(100),LOCAL(100)
 
      IF(N.NE.1) GO TO 1
      LAMDA(1)=A(1,1)
      LAMDA(2)=0.D0
      VECR(1,1) = 1.0D0
      VECI(1,1) = 0.0D0
      INDIC(1) = 2
      GO TO 25
 
    1 CALL SKALEI(N,NM,A,VECI,PRFACT,ENORM)
 
C THE COMPUTATION OF THE EIGENVALUES OF THE NORMALISED
C MATRIX.
 
      EX = DEXP(-T*DLOG(2.0D0))
      CALL HESQR1(N,NM,A,VECI,LAMDA,SUBDIA,INDIC,EPS,EX)
 
C THE POSSIBLE DECOMPOSITION OF THE UPPER-HESSENBERG MATRIX
C INTO THE SUBMATRICES OF LOWER ORDER IS INDICATED IN THE
C ARRAY LOCAL. THE DECOMPOSITION OCCURS WHEN SOME
C SUBDIAGONAL ELEMENTS ARE IN MODULUS LESS THAN A SMALL
C POSITIVE NUMBER EPS DEFINED IN THE SUBROUTINE HESQR. THE
C AMOUNT OF WORK IN THE EIGENVECTOR PROBLEM MAY BE
C DIMINISHED IN THIS WAY.
 
      J = N
      I = 1
      LOCAL(1) = 1
      IF(J.EQ.1) GO TO 4
    2 IF(DABS(SUBDIA(J-1)).GT.EPS) GO TO 3
      I = I+1
      LOCAL(I) = 0
    3 J = J-1
      LOCAL(I) = LOCAL(I)+1
      IF(J.NE.1) GO TO 2
 
C THE EIGENVECTOR PROBLEM.
 
    4 K = 1
      KON = 0
      L = LOCAL(1)
      M = N
      DO 10 I=1,N
      IVEC = N-I+1
      IF(I.LE.L) GO TO 5
      K = K+1
      M = N-L
      L = L+LOCAL(K)
    5 IF(INDIC(IVEC).EQ.0) GO TO 10
      IF (LAMDA(2*IVEC) .NE. 0.D0) GO TO 8
 
C TRANSFER OF AN UPPER-HESSENBERG MATRIX OF ORDER M FROM
C THE ARRAYS VECI AND SUBDIA INTO THE ARRAY A.
 
      DO 7 K1=1,M
      DO 6 L1=K1,M
    6 A(K1,L1) = VECI(K1,L1)
      IF(K1.EQ.1) GO TO 7
      A(K1,K1-1) = SUBDIA(K1-1)
    7 CONTINUE
 
C THE COMPUTATION OF THE REAL EIGENVECTOR IVEC OF THE UPPER-
C HESSENBERG MATRIX CORRESPONDING TO THE REAL EIGENVALUE
C EVR(IVEC).
 
      CALL REALV1
     *   (N,NM,M,IVEC,A,VECR,LAMDA,IWORK,WORK,INDIC,EPS,EX)
      GO TO 10
 
C THE COMPUTATION OF THE COMPLEX EIGENVECTOR IVEC OF THE
C UPPER-HESSENBERG MATRIX CORRESPONDING TO THE COMPLEX
C EIGENVALUE EVR(IVEC) + I*EVI(IVEC). IF THE VALUE OF KON IS
C NOT EQUAL TO ZERO THEN THIS COMPLEX EIGENVECTOR HAS
C ALREADY BEEN FOUND FROM ITS CONJUGATE.
 
    8 IF(KON.NE.0) GO TO 9
      KON = 1
      CALL CMPV1 (N,NM,M,IVEC,A,VECR,VECI,LAMDA,INDIC,
     *            IWORK,SUBDIA,WORK1,WORK2,WORK,EPS,EX)
      GO TO 10
    9 KON = 0
   10 CONTINUE
 
C THE RECONSTRUCTION OF THE MATRIX USED IN THE REDUCTION OF
C MATRIX A TO AN UPPER-HESSENBERG FORM BY HOUSEHOLDER METHOD
 
      DO 12 I=1,N
      DO 11 J=I,N
      A(I,J) = 0.0D0
   11 A(J,I) = 0.0D0
   12 A(I,I) = 1.0D0
      IF(N.LE.2) GO TO 15
      M = N-2
      DO 14 K=1,M
      L = K+1
      DO 14 J=2,N
      D1 = 0.0D0
      DO 13 I=L,N
      D2 = VECI(I,K)
   13 D1 = D1+D2*A(J,I)
      DO 14 I=L,N
   14 A(J,I) = A(J,I)-VECI(I,K)*D1
 
C THE COMPUTATION OF THE EIGENVECTORS OF THE ORIGINAL NON-
C SCALED MATRIX.
 
   15 KON = 1
      DO 24 I=1,N
      I2=2*I
      I21=I2-1
      L = 0
      IF(LAMDA(I2) .EQ. 0.D0) GO TO 16
      L = 1
      IF(KON.EQ.0) GO TO 16
      KON = 0
      GO TO 24
   16 DO 18 J=1,N
      D1 = 0.0D0
      D2 = 0.0D0
      DO 17 K=1,N
      D3 = A(J,K)
      D1 = D1+D3*VECR(K,I)
      IF(L.EQ.0) GO TO 17
      D2 = D2+D3*VECR(K,I-1)
   17 CONTINUE
      WORK(J) = D1/PRFACT(J)
      IF(L.EQ.0) GO TO 18
      SUBDIA(J) = D2/PRFACT(J)
   18 CONTINUE
 
C THE NORMALISATION OF THE EIGENVECTORS AND THE COMPUTATION
C OF THE EIGENVALUES OF THE ORIGINAL NON-NORMALISED MATRIX.
 
      IF(L.EQ.1) GO TO 21
      D1 = 0.0D0
      DO 19 M=1,N
   19 D1 = D1+WORK(M)**2
      D1 = DSQRT(D1)
      DO 20 M=1,N
      VECI(M,I)=0.D0
   20 VECR(M,I) = WORK(M)/D1
      LAMDA(I21)=LAMDA(I21)*ENORM
      GO TO 24
 
   21 KON = 1
      LAMDA(I21)=LAMDA(I21)*ENORM
      LAMDA(I21-2)=LAMDA(I21)
      LAMDA(I2)=LAMDA(I2)*ENORM
      LAMDA(I2-2)=-LAMDA(I2)
      R = 0.0D0
      DO 22 J=1,N
      R1 = WORK(J)**2+SUBDIA(J)**2
      IF(R.GE.R1) GO TO 22
      R = R1
      L = J
   22 CONTINUE
      D3 = WORK(L)
      R1 = SUBDIA(L)
      DO 23 J=1,N
      D1 = WORK(J)
      D2 = SUBDIA(J)
      VECR(J,I) = (D1*D3+D2*R1)/R
      VECI(J,I)=(D2*D3-D1*R1)/R
      VECR(J,I-1) = VECR(J,I)
   23 VECI(J,I-1)=-VECI(J,I)
   24 CONTINUE
 
C  CORRECT STORAGE IN AC ARRAY (MODAL MATRIX)
 
   25 NN=N+1
      DO 251 I=1,N
      DO 251 J=1,N
  251 A(I,J)=VECI(I,J)
      DO 26 JJ=1,N
      J=NN-JJ
      DO 26 II=1,N
      I=NN-II
   26 AC(I,J)=DCMPLX(VECR(I,J),A(I,J))
      RETURN
 
      END
