CSECT   HESQR1
 
      SUBROUTINE HESQR1(N,NM,A,H,LAMDA,SUBDIA,INDIC,EPS,EX)
 
C THIS SUBROUTINE FINDS ALL THE EIGENVALUES OF A REAL
C GENERAL MATRIX. THE ORIGINAL MATRIX A OF ORDER N IS
C REDUCED TO THE UPPER-HESSENBERG FORM H BY MEANS OF
C SIMILARITY TRANSFORMATIONS (HOUSEHOLDER METHOD). THE MATRIX
C H IS PRESERVED IN THE UPPER HALF OF THE ARRAY H AND IN THE
C ARRAY SUBDIA. THE SPECIAL VECTORS USED IN THE DEFINITION
C OF THE HOUSEHOLDER TRANSFORMATION MATRICES ARE STORED IN
C THE LOWER PART OF THE ARRAY H.
C NM IS THE FIRST DIMENSION OF THE ARRAYS A AND H. NM MUST
C BE EQUAL TO OR GREATER THAN N.
C THE REAL PARTS OF THE N EIGENVALUES WILL BE FOUND IN THE
C FIRST N PLACES OF THE ARRAY EVR, AND
C THE IMAGINARY PARTS IN THE FIRST N PLACES OF THE ARRAY EVI
C THE ARRAY INDIC INDICATES THE SUCCESS OF THE ROUTINE AS
C FOLLOWS
C     VALUE OF INDIC(I)     EIGENVALUE I
C            0                NOT FOUND
C            1                  FOUND
C EPS IS A SMALL POSITIVE NUMBER THAT NUMERICALLY REPRESENTS
C ZERO IN THE PROGRAM. EPS = (EUCLIDIAN NORM OF H)*EX, WHERE
C EX = 2**(-T). T IS THE NUMBER OF BINARY DIGITS IN THE
C MANTISSA OF A FLOATING POINT NUMBER.
 
C REDUCTION OF THE MATRIX A TO AN UPPER-HESSENBERG FORM H.
C THERE ARE N-2 STEPS.
 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z),INTEGER(I-N)
 
      DOUBLE PRECISION LAMDA (NM)
      DIMENSION        A(NM,1),H(NM,1),SUBDIA(NM),INDIC(NM)
 
      IF(N-2) 14,1,2
    1 SUBDIA(1) = A(2,1)
      GO TO 14
    2 M = N-2
      DO 12 K=1,M
      L = K+1
      S = 0.0D0
      DO 3 I=L,N
      H(I,K) = A(I,K)
    3 S = S+ABS(A(I,K))
      IF(S.NE.ABS(A(K+1,K))) GO TO 4
      SUBDIA(K) = A(K+1,K)
      H(K+1,K) = 0.0D0
      GO TO 12
    4 SR2 = 0.0D0
      DO 5 I=L,N
      SR = A(I,K)/S
      A(I,K) = SR
    5 SR2 = SR2+SR*SR
      SR = SQRT(SR2)
      IF(A(L,K).LT.0.0D0) GO TO 6
      SR = -SR
    6 SR2 = SR2-SR*A(L,K)
      A(L,K) = A(L,K)-SR
      H(L,K) = H(L,K)-SR*S
      SUBDIA(K) = SR*S
      X = S*SQRT(SR2)
      DO 7 I=L,N
      H(I,K) = H(I,K)/X
    7 SUBDIA(I) = A(I,K)/SR2
 
C PREMULTIPLICATION BY THE MATRIX PR.
 
      DO 9 J=L,N
      SR = 0.0D0
      DO 8 I=L,N
    8 SR = SR+A(I,K)*A(I,J)
      DO 9 I=L,N
    9 A(I,J) = A(I,J)-SUBDIA(I)*SR
 
C POSTMULTIPLICATION BY THE MATRIX PR.
 
      DO 11 J=1,N
      SR = 0.0D0
      DO 10 I=L,N
   10 SR = SR+A(J,I)*A(I,K)
      DO 11 I=L,N
   11 A(J,I) = A(J,I)-SUBDIA(I)*SR
   12 CONTINUE
      DO 13 K=1,M
   13 A(K+1,K) = SUBDIA(K)
 
C TRANSFER OF THE UPPER HALF OF THE MATRIX A INTO THE
C ARRAY H AND THE CALCULATION OF THE SMALL POSITIVE NUMBER
C EPS.
 
      SUBDIA(N-1) = A(N,N-1)
   14 EPS = 0.0D0
      DO 15 K=1,N
      INDIC(K) = 0
      IF (K.NE.N) EPS = EPS+SUBDIA(K)**2
      DO 15 I=K,N
      H(K,I) = A(K,I)
   15 EPS = EPS+A(K,I)**2
      EPS = EX*SQRT(EPS)
 
C THE QR ITERATIVE PROCESS. THE UPPER-HESSENBERG MATRIX H IS
C REDUCED TO THE UPPER-MODIFIED TRIANGULAR FORM.
 
C DETERMINATION OF THE SHIFT OF ORIGIN FOR THE FIRST STEP OF
C THE QR ITERATIVE PROCESS.
 
      SHIFT = A(N,N-1)
      IF(N.LE.2) SHIFT = 0.0D0
      IF(A(N,N).NE.0.0D0) SHIFT = 0.0D0
      IF(A(N-1,N).NE.0.0D0) SHIFT = 0.0D0
      IF(A(N-1,N-1).NE.0.0D0) SHIFT = 0.0D0
      M = N
      NS = 0
      MAXST = N*10
 
C TESTING IF THE UPPER HALF OF THE MATRIX IS EQUAL TO ZERO.
C IF IT IS EQUAL TO ZERO THE QR PROCESS IS NOT NECESSARY.
 
      DO 16 I=2,N
      DO 16 K=I,N
      IF(A(I-1,K).NE.0.0D0) GO TO 18
   16 CONTINUE
      DO 17 I=1,N
      I2=2*I
      INDIC(I) = 1
      LAMDA(I2-1)=A(I,I)
   17 LAMDA(I2)=0.D0
      GO TO 37
 
C START THE MAIN LOOP OF THE QR PROCESS.
 
   18 K = M-1
      M1 = K
      I = K
 
C FIND ANY DECOMPOSITIONS OF THE MATRIX.
C JUMP TO 34 IF THE LAST SUBMATRIX OF THE DECOMPOSITION IS
C OF THE ORDER ONE.
C JUMP TO 35 IF THE LAST SUBMATRIX OF THE DECOMPOSITION IS
C OF THE ORDER TWO.
 
      IF(K) 37,34,19
   19 IF(ABS(A(M,K)).LE.EPS) GO TO 34
      IF(M-2.EQ.0) GO TO 35
   20 I = I-1
      IF(ABS(A(K,I)).LE.EPS) GO TO 21
      K = I
      IF(K.GT.1) GO TO 20
   21 IF(K.EQ.M1) GO TO 35
 
C TRANSFORMATION OF THE MATRIX OF THE ORDER GREATER THAN TWO.
 
      S = A(M,M)+A(M1,M1)+SHIFT
      SR = A(M,M)*A(M1,M1)-A(M,M1)*A(M1,M)+0.25D0*SHIFT**2
      A(K+2,K) = 0.0D0
 
C CALCULATE X1, Y1, Z1, FOR THE SUBMATRIX OBTAINED BY THE
C DECOMPOSITION.
 
      X = A(K,K)*(A(K,K)-S)+A(K,K+1)*A(K+1,K)+SR
      Y = A(K+1,K)*(A(K,K)+A(K+1,K+1)-S)
      R = ABS(X)+ABS(Y)
      IF (R.EQ.0.0D0) SHIFT = A(M,M-1)
      IF (R.EQ.0.0D0) GO TO 21
      Z = A(K+2,K+1)*A(K+1,K)
      SHIFT = 0.0D0
      NS = NS+1
 
C THE LOOP FOR ONE STEP OF THE QR PROCESS.
 
      DO 33 I=K,M1
      IF(I.EQ.K) GO TO 22
 
C CALCULATE XR, YR, ZR.
 
      X = A(I,I-1)
      Y = A(I+1,I-1)
      Z = 0.0D0
      IF (I+2.GT.M) GO TO 22
      Z = A(I+2,I-1)
   22 SR2 = ABS(X)+ABS(Y)+ABS(Z)
      IF (SR2.EQ.0.0D0) GO TO 23
      X = X/SR2
      Y = Y/SR2
      Z = Z/SR2
   23 S = SQRT(X*X+Y*Y+Z*Z)
      IF(X.LT.0.0D0) GO TO 24
      S = -S
   24 IF(I.EQ.K) GO TO 25
      A(I,I-1) = S*SR2
   25 IF(SR2.NE.0.0D0) GO TO 26
      IF(I+3.GT.M) GO TO 33
      GO TO 32
   26 SR = 1.0D0-X/S
      S = X-S
      X = Y/S
      Y = Z/S
 
C PREMULTIPLICATION BY THE MATRIX PR.
 
      DO 28 J=I,M
      S = A(I,J)+A(I+1,J)*X
      IF(I+2.GT.M) GO TO 27
      S = S+A(I+2,J)*Y
   27 S = S*SR
      A(I,J) = A(I,J)-S
      A(I+1,J) = A(I+1,J)-S*X
      IF(I+2.GT.M) GO TO 28
      A(I+2,J) = A(I+2,J)-S*Y
   28 CONTINUE
 
C POSTMULTIPLICATION BY THE MATRIX PR.
 
      L = I+2
      IF(I.LT.M1) GO TO 29
      L = M
   29 DO 31 J=K,L
      S = A(J,I)+A(J,I+1)*X
      IF(I+2.GT.M) GO TO 30
      S = S+A(J,I+2)*Y
   30 S = S*SR
      A(J,I) = A(J,I)-S
      A(J,I+1) = A(J,I+1)-S*X
      IF(I+2.GT.M) GO TO 31
      A(J,I+2) = A(J,I+2)-S*Y
   31 CONTINUE
      IF(I+3.GT.M) GO TO 33
      S = -A(I+3,I+2)*Y*SR
   32 A(I+3,I) = S
      A(I+3,I+1) = S*X
      A(I+3,I+2) = S*Y+A(I+3,I+2)
   33 CONTINUE
 
      IF(NS.GT.MAXST) GO TO 37
      GO TO 18
 
C COMPUTE THE LAST EIGENVALUE.
 
   34 LAMDA(2*M-1)=A(M,M)
      LAMDA(2*M)=0.D0
      INDIC(M) = 1
      M = K
      GO TO 18
 
C COMPUTE THE EIGENVALUES OF THE LAST 2X2 MATRIX OBTAINED BY
C THE DECOMPOSITION.
 
   35 R = 0.5D0*(A(K,K)+A(M,M))
      S = 0.5D0*(A(M,M)-A(K,K))
      S = S*S+A(K,M)*A(M,K)
      INDIC(K) = 1
      INDIC(M) = 1
      IF(S.LT.0.0D0) GO TO 36
      T = SQRT(S)
      LAMDA(2*K-1)=R-T
      LAMDA(2*M-1)=R+T
      LAMDA(2*K)=0.D0
      LAMDA(2*M)=0.D0
      M = M-2
      GO TO 18
   36 T = SQRT(-S)
      LAMDA(2*K-1)=R
      LAMDA(2*K)=T
      LAMDA(2*M-1)=R
      LAMDA(2*M)=-T
      M = M-2
      GO TO 18
 
   37 RETURN
 
      END
