CSECT   LINSOL                          72.110  00.39.08
 
      SUBROUTINE LINSOL (ANAME,AMAT,B,IP,N,ICNTRL,IERR)
 
C     THIS SUBROUTINE GENERATES THE SOLUTION FOR A LINEAR SYSTEM OF
C     EQUATIONS.  THIS SOLUTION IS BY TRIANGULARIZATION.  THAT IS,
C     THE MATRIX IS DECOMPOSED INTO A PRODUCT OF A LOWER TRIANGULAR
C     MATRIX AND AN UPPER TRIANGULAR MATRIX WITH ONES ON THE DIAGONAL.
C     THE SOLUTION IS THEN GENERATED USING FORWARD AND BACK
C     SUBSTITUTION.
C     THE ARGUMENTS ARE:
C         ANAME-  MATRIX NAME
C         AMAT-   THE MATRIX
C         B-      SOLUTION VECTOR
C         IP-     PERMUTATION VECTOR (I.E.,MATRIX ROW ORDER AFTER
C                 PIVOTING).
C         N-      DIMENSION OF THE MATRIX.
C         ICNTRL- A CONTROL THAT HAS VALUES:
C                 = 1 , GENERATE UPPER AND LOWER MATRICES ONLY ON THIS
C                       PASS.
C                 =2  , GENERATE SOLUTION ONLY ON THIS PASS.
C                 =3  , PREFORM BOTH OF THE ABOVE.
C         IERR-   ERROR INDICATOR
C                  = 0 , NORMAL RETURN
C                  =10 , MATRIX IS SINGULAR
 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
 
      COMMON /TAPES/  NOUTTP
      DIMENSION AMAT(N,N),B(N),IP(N)
 
      DATA ZERO /0D0/
 
      IERR=0
 
C IS DIMENSION OF MATRIX = ONE
 
      IF (N.GT.1) GO TO 100
 
C PROVIDE SOLUTION FOR A MATRIX OF DIMENSION ONE IF REQUIRED
 
      IF (ICNTRL.EQ.1) RETURN
      B(1)=B(1)/AMAT(1,1)
      RETURN
 
C IS SOLUTION ONLY REQUIRED?
 
  100 IF (ICNTRL.EQ.2) GO TO 300
 
C LOAD THE ORIGINAL ROW ORDER INTO IP
 
      DO 105 I=1,N
  105 IP(I)=I
 
C COMPUTE THE UPPER AND LOWER MATRIX VALUES.
 
      ICHNGE=0
      DO 200 I=1,N
      IROW=0
      CMAX=ZERO
      LCOL=I-1
      IF (LCOL.GT.0) GO TO 120
      DO 110 K=1,N
      IF (ABS(AMAT(K,1)).LE.CMAX) GO TO 110
      CMAX=ABS(AMAT(K,1))
      IROW=K
  110 CONTINUE
      GO TO 155
  120 LROW=IP(LCOL)
      DO 150  K=I,N
      I1=IP(K)
      AMULT=AMAT(I1,LCOL)
      IF (AMULT.EQ.ZERO) GO TO 140
      DO 130 J=I,N
  130 AMAT(I1,J)=AMAT(I1,J)-AMULT*AMAT(LROW,J)
  140 IF (ABS(AMAT(I1,I)).LE.CMAX) GO TO 150
      IROW=K
      CMAX=ABS(AMAT(I1,I))
  150 CONTINUE
 
C IS MATRIX SINGULAR?
 
  155 IF (IROW.EQ.0) GO TO 210
      K=IP(IROW)
      IF (IROW.EQ.I) GO TO 170
      ICHNGE=1
      IP(IROW)=IP(I)
      IP(I)=K
  170 IF (I.EQ.N) GO TO 200
      DIAG=AMAT(K,I)
      ISTRT=I+1
      DO 180 J=ISTRT,N
  180 AMAT(K,J)=AMAT(K,J)/DIAG
  200 CONTINUE
 
C IS SOLUTION REQUIRED?
 
      IF (ICNTRL.EQ.3) GO TO 300
      RETURN
 
C WRITE ERROR MESSAGE
 
  210 IERR=10
      WRITE(NOUTTP,220) ANAME
  220 FORMAT('0LINSOL: MATRIX ',A4,' IS SINGULAR',//)
      RETURN
 
C PERFORM FORWARD SUBSTITUTION
 
  300 I=IP(1)
      B(I)=B(I)/AMAT(I,1)
      DO 400 K=2,N
      I=IP(K)
      NEND=K-1
      BSAVE=B(I)
      DO 350 J=1,NEND
      L=IP(J)
  350 BSAVE=BSAVE-AMAT(I,J)*B(L)
  400 B(I)=BSAVE/AMAT(I,K)
 
C PERFORM BACK SUBSTITUTION
 
      I=N-1
      NEND=I
      DO 500 K=1,NEND
      ISTRT=I+1
      L=IP(I)
      BSAVE=B(L)
      DO 450 J=ISTRT,N
      M=IP(J)
  450 BSAVE=BSAVE-AMAT(L,J)*B(M)
      B(L)=BSAVE
  500 I=I-1
 
C PUT SOLUTION VECTOR INTO CORRECT ORDER
 
      IF (ICHNGE.EQ.0) RETURN
      DO 560 I=1,NEND
      IF(IP(I)-I) 510,560,540
  510 K=IP(I)
  520 IF(IP(K)-I) 530,560,550
  530 K=IP(K)
      GO TO 520
  540 K=I
  550 SAVE=B(I)
      K=IP(K)
      B(I)=B(K)
      B(K)=SAVE
  560 CONTINUE
  800 RETURN
 
      END
