CSECT   MONTE
C       1 MAY 73  (WNR)
 
      SUBROUTINE MONTE (KWAY)
 
C     NMCSTS = NUMBER OF MONTE CARLO SETS OF VARIABLES
C
C     KMCSTS(4,NMCSTS) - MONTE CARLO SET TABLE
C     KMCSTS(1,I) = INDEX TO DEPENDENT VARIABLE TABLE 'KMCDEP'
C     KMCSTS(2,I) = NUMBER OF DEPENDENT VARIABLES FOR SET 'I'
C     KMCSTS(3,I) = INDEX TO INDEPENDENT VARIABLE TABLE 'KMCIND'
C     KMCSTS(4,I) = NUMBER OF INDEPENDENT VARIABLES FOR SET 'I'
C
C     KMCDEP(6,NDEP)-DEPENDENT VARIABLE TABLE
C     KMCDEP(1,I)=INDEX TO DEPENDENT VARIABLE NAME IN NAMES TABLE
C     KMCDEP(2,I)=FUNCTION TYPE  1=VOLTAGE,2=CURRENT,3=VALUE,4=DEF PARAM
C     KMCDEP(3,I)=DUMMY ENTRY
C     KMCDEP(4,I)=DUMMY ENTRY
C     KMCDEP(5,I)=DUMMY ENTRY
C     KMCDEP(6,I)=DUMMY ENTRY
C
C     KMCIND(6,NIND)-INDEPENDENT VARIABLE TABLE
C     KMCIND(1,I)=DUMMY ENTRY
C     KMCIND(2,I)=DUMMY ENTRY
C     KMCIND(3,I)=DUMMY ENTRY
C     KMCIND(4,I)=DUMMY ENTRY
C     KMCIND(5,I)=INDEX TO INDEP. VARIABLE IN ELEMENT OR DEF. PAR. TABLE
C     KMCIND(6,I)=TYPE (3=ELEMENT, 4=DEFINED PARAMETER)
 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
 
      LOGICAL FIRST
 
      COMMON /ADJDAT/ NIS,NSETS,NOPSTS,NWCSTS,NMCSTS
      COMMON /MONTA/  KMCSTS(4,1)
      COMMON /MONTB/  KMCDEP(6,1)
      COMMON /MONTC/  KMCIND(6,1)
      COMMON /OPTEMP/ U(1)
      COMMON /CNTRLS/ CTROLS(160)
         EQUIVALENCE (XNMCPS,CTROLS(121)),(XNELTS,CTROLS(110)),
     $               (XIRNO1,CTROLS(122)),(XIRNO2,CTROLS(123)),
     $               (XDISTR,CTROLS(125)),(XTRNIC,CTROLS(55)),
     $               (XNDP,  CTROLS(108)),(XLSW,  CTROLS(111)),
     $               (XLSWP, CTROLS(112)),(XICPRT,CTROLS(115)),
     $               (XICDBG,CTROLS(116)),(XLMCD, CTROLS(124))
      COMMON /ELNAMS/ ELS(1)
      COMMON /ELIMIT/ EBOUND(7,1)
      COMMON /DPNAMS/ DPS(1)
      COMMON /PLIMIT/ PBOUND(5,1)
      COMMON /PRMTRS/ PRAMTR(1)
      COMMON /NAMES/  VAL(1)
      COMMON /OSTATS/ STOUT(1)
      COMMON /SAVERN/ KRAND
      COMMON /TAPES/  NOUTTP,iskip(11),nout
      CHARACTER       TYPE(2)*5, DIST(2)*8, TIND*12, TDEP*12
 
      DATA IRAND /127263527/
      DATA DIST /'GAUSSIAN','UNIFORM'/
      DATA TIND /' INDEPENDENT'/
      DATA TDEP /'  DEPENDENT'/
      DATA TYPE /'(IND)','(DEP)'/
 
C     TURN OFF NORMAL IC OUTPUT UNLESS IC DEBUG REQUESTED
 
      XICPRT = 1.D0
      IF (nint(XICDBG) .NE. 0) XICPRT = 0.D0
 
C     EBOUND (ELEMENT) AND PBOUND (DEFINED PARAMETER) ENTRIES -
C     CELL 1 IS NOMINAL INPUT
C     CELL 2 IS MINIMUM INPUT
C     CELL 3 IS MAXIMUM INPUT
C     CELL 4 IS INPUT MEAN
C     CELL 5 IS INPUT STANDARD DEVIATION
C     CELL 6 IS NOMINAL (ELEMENT) VOLTAGE
C     CELL 7 IS NOMINAL (ELEMENT) CURRENT
C
C     STOUT(L,6)  TABULATED ENTRIES FOR OUTPUT OF IND AND DEP VARIABLES
C     STOUT(L,1)  VARIABLE NAMES
C     STOUT(L,2)  NOMINAL VALUES
C     STOUT(L,3)  MINIMUM VALUES
C     STOUT(L,4)  MAXIMUM VALUES
C     STOUT(L,5)  MEAN
C     STOUT(L,6)  STANDARD DEVIATION
C
C     WHEN MONTE CARLO DETAILS ARE REQUESTED, STOUT IS DIMENSIONED
C     (L,9) AND THE SPACE FROM (1,3) THROUGH (L,9) IS ALSO USED AS
C     OUTPUT BUFFER
C
C     LDIST IS DISTRIBUTION SWITCH
C          1 FOR GAUSSIAN DISTRIBUTION
C          2 FOR UNIFORM DISTRIBUTION
 
      LDIST = nint(XDISTR) + 1
 
C     K IS NUMBER OF DEFINED PARAMETERS
C     M IS NUMBER OF ITERATIONS
C     N IS NUMBER OF ELEMENTS
 
      K = nint(XNDP)
      M = nint(XNMCPS)
      N = nint(XNELTS)
 
C     CREATE INPUT STATISTICS
 
      WRITE (NOUTTP,4640)
 4640 FORMAT ('1_____________________',/,
     *        '+INPUTS TO MONTE CARLO',/,
     *        '0NAME',10X,'NOMINAL',13X,'LBOUND',14X,'UBOUND',14X,
     *        'MEAN',16X,'SIGMA')
 
C     ELEMENT SCAN
 
      IF (nint(XLSW) .ne. 0) then
         WRITE (NOUTTP,*)
         DO 120 J=1,N
            EBOUND(4,J) = (EBOUND(3,J) + EBOUND(2,J))/2.D0
            EBOUND(5,J) = (EBOUND(3,J) - EBOUND(2,J))/6.D0
            IF (EBOUND(5,J).NE.0.D0) WRITE (NOUTTP,4660)ELS(J),
     *         (EBOUND(I,J),I = 1,5)
 4660       FORMAT (1X,A6,1P,5E20.7)
  120    CONTINUE
      end if
 
C     DEFINED PARAMETER SCAN
 
      IF (nint(XLSWP) .ne. 0) then
         WRITE (NOUTTP,*)
         DO 160 J = 1,K
            PBOUND(4,J) = (PBOUND(3,J) + PBOUND(2,J))/2.D0
            PBOUND(5,J) = (PBOUND(3,J) - PBOUND(2,J))/6.D0
            IF (PBOUND(5,J).NE.0.D0) WRITE (NOUTTP,4660)DPS(J),
     *         (PBOUND(I,J),I = 1,5)
  160       CONTINUE
      end if
 
C     INITIALIZE RANDOM NUMBERS
C     IF TRANSMITTED VALUE IS
C          POSITIVE, USE TRANSMITTED VALUE
C          NEGATIVE, USE DEFAULT VALUE IRAND
C          ZERO,     USE KRAND IF INITIALIZED, OTHERWISE USE IRAND
 
      JRAND = nint(XIRNO1) + nint(XIRNO2)
      IF (JRAND) 220,200,240
  200 IF (KRAND .LE. 0)  GO TO 220
      JRAND = KRAND
      GO TO 240
  220 JRAND = IRAND
  240 CONTINUE

      DO 1000 ISET = 1,NMCSTS
         IDEP = KMCSTS(1,ISET)
         NDEP = KMCSTS(2,ISET)
 
C     CHECK DEPENDENT VARIABLES
 
         IF (NDEP .LT. 1) then
            WRITE (NOUTTP,'(''0NO DEPENDENT VARIABLE SPECIFIED'')')
            GO TO 6000
         end if
 
         INDPV = KMCSTS(3,ISET)
         NINDV = KMCSTS(4,ISET)
 
C     CHECK INDEPENDENT VARIABLES
 
         IF (NINDV .LT. 1) then
            WRITE (NOUTTP,'(''0NO INDEPENDENT VARIABLE SPECIFIED'')')
            GO TO 6000
         end if

C      NINDP is switch for output of 'DEPENDENT/INDEPENDENT' string
 
         NINDP = NINDV + 1
 
C     L IS NUMBER OF ITEMS TABULATED
 
         L = NDEP + NINDV
 
C     GET VARIABLE NAMES
 
         CALL MCNAME (NINDV,KMCIND(1,INDPV),NDEP,KMCDEP(1,IDEP),STOUT)
 
C     SAVE NOMINAL VALUES
 
         CALL EPRST
         CALL MCLOAD (NINDV,KMCIND(1,INDPV),NDEP,KMCDEP(1,IDEP),
     *               STOUT(L + 1))
         WRITE (NOUTTP,4670) JRAND,DIST(LDIST)
 4670    FORMAT ('0'/'0INITIAL RANDOM NUMBER = ',I12 /
     *           '0DISTRIBUTION IS ', A)
         FIRST = .TRUE.
 
C     INITIALIZE SUCCESSFUL LOOP COUNTER
 
         REWIND NOUT
         IT = 0
 
C     MAIN COMPUTE LOOP
 
         DO 600 IMLC = 1,M
 
C     LOAD INDEPENDENT VARIABLES
 
            IZZ = INDPV - 1
            DO 320 J = 1,NINDV
               IZZ = IZZ + 1
               CALL RANDOM (LDIST,JRAND,GDRN)
               IX = KMCIND(5,IZZ)
               IY = KMCIND(6,IZZ)
               IF (IY .EQ. 4) then
                  PRAMTR(IX) = PBOUND(4,IX) + PBOUND(5,IX)*GDRN
                else
                  VAL(IX) = EBOUND(4,IX) + EBOUND(5,IX)*GDRN
               end if
  320       CONTINUE
 
C     DC SOLUTION
 
            KWAY = 0
            CALL SIMDC (KWAY)
 
C     GET CURRENT VALUES
 
            CALL MCLOAD (NINDV,KMCIND(1,INDPV),NDEP,KMCDEP(1,IDEP),U)
 
C     CHECK FOR NORMAL RETURN
 
            IF (KWAY .ne. 0) then

C     IC FAILED:

  640          WRITE (NOUTTP,4610) IMLC
 4610          FORMAT ('0INITIAL CONDITIONS FAILURE AT RUN ',I4)
 
C      WRITE OUT INDEPENDENT VARIABLE VALUES
 
               WRITE (NOUTTP,4900)
 4900          FORMAT ('0INDEPENDENT VARIABLES AT FAILURE ',//
     *                 3X,'NAME',15X,'VALUE',/)
               DO 660 J = 1,NINDV
                  WRITE (NOUTTP,4680)STOUT(J),U(J)
  660          CONTINUE
               GO TO 600
            END IF

C     INCREMENT COUNTER
 
  400       IT = IT + 1
 
C     SAVE VARIABLE VALUES
 
            WRITE (NOUT) IT, (U(J),J = 1,L)
 
C     CHECK MONTE CARLO DETAILS REQUEST
   
            IF (nint(XLMCD) .ne. 0) then
               IF (FIRST) WRITE (NOUTTP,'(''0MONTE CARLO DETAILS:'')')
               FIRST = .FALSE.
               WRITE (NOUTTP,4800) IT
 4800          FORMAT ('0ITERATION ',8I13)
               ITYPE = 1
               DO 500 I=1,L
                  IF (I .EQ. NINDP) ITYPE = 2
                  WRITE (NOUTTP,4810) STOUT(I),TYPE(ITYPE),U(I)
 4810             FORMAT (1X,A8,1X,A5,8F13.6)
  500          CONTINUE
            END IF
  600    CONTINUE

         WRITE (NOUTTP,4620) IT
 4620    FORMAT ('0NORMAL MONTE CARLO TERMINATION AFTER ',I4,
     *           ' ITERATIONS')
         ENDFILE NOUT
         REWIND NOUT
 
C     SAVE LAST-USED RANDOM NUMBER
 
         WRITE (NOUTTP,4630) JRAND
 4630    FORMAT ('0FINAL RANDOM NUMBER = ',I12)
         KRAND = JRAND
         if (it.le.0) go to 1000
 
C     INITIALIZE STATISTICAL OUTPUTS
 
         READ (NOUT) I,(U(J),J = 1,L)
         DO 720 J = 1,L
            VALC = U(J)
 
C     COMPUTE DEVIATIONS FROM NOMINAL VALUES
 
            J1L = J + L
            J2L = J1L + L
            J3L = J2L + L
            J4L = J3L + L
            J5L = J4L + L
            DVAL = VALC - STOUT(J1L)
            STOUT(J2L) = VALC
            STOUT(J3L) = VALC
            STOUT(J4L) = DVAL
            STOUT(J5L) = DVAL**2
  720    continue
         IF (IT .ge. 2) then
            DO 760 I = 2,IT
CND:  THIS CHANGE AVOIDS THE REDEFINITION OF THE DO VARIABLE I:
CND:  READ (NOUT) I, (U(J), J=1,L)
               READ (NOUT) IDUMMY, (U(J), J=1,L)
               DO 740 J = 1,L
                  VALC = U(J)
                  J1L = J + L
                  J2L = J1L + L
                  J3L = J2L + L
                  J4L = J3L + L
                  J5L = J4L + L
                  DVAL = VALC - STOUT(J1L)
                  IF (VALC .LT. STOUT(J2L)) STOUT(J2L) = VALC
                  IF (VALC .GT. STOUT(J3L)) STOUT(J3L) = VALC
                  STOUT(J4L) = STOUT(J4L) + DVAL
                  STOUT(J5L) = STOUT(J5L) + DVAL**2
  740          continue
  760       CONTINUE
         end if
         DO 800 J = 1,L
            J1L = J + L
            J4L = J1L + 3*L
            J5L = J4L + L
            DSUM = STOUT(J4L)
            AVE = DSUM/dble(it)
            STOUT(J4L) = AVE + STOUT(J1L)
            OSIG = 0.D0
            IF (IT .GT. 1)
     *         OSIG = SQRT(MAX((STOUT(J5L) - DSUM*AVE)/dble(IT-1),0.D0))
            STOUT(J5L) = OSIG
  800    continue
         REWIND NOUT
         WRITE (NOUTTP,'(''0___________________'',/,
     *                   ''+OBSERVED STATISTICS'')')
         WRITE (NOUTTP,4600) TIND
 4600    FORMAT (//,A12,9X,'NOMINAL',13X,'MINIMUM',13X,'MAXIMUM',
     *    13X,'SAMPLE',14X,'SAMPLE',/,2X,'VARIABLE',12X,'VALUE',
     *    15X,'VALUE',15X,'VALUE',15X,'MEAN',16X,'SIGMA',/)
         DO 820 J = 1,L
            IF (J .EQ. NINDP) WRITE (NOUTTP,4600) TDEP
            I1 = J
            I2 = I1 + 5*L
            WRITE (NOUTTP,4680) (STOUT(I),I = I1,I2,L)
 4680       FORMAT (2X,A8,2X,1p,5(e20.7))
  820    CONTINUE
 1000 CONTINUE
 6000 CONTINUE 

C     TURN ON NORMAL IC OUTPUT
 
      XICPRT = 0.D0
      RETURN
 
      END
