CSECT   MXMULT
 
      SUBROUTINE MXMULT(X,IXZERO,MRX,MCX,Y,IYZERO,IYTRNS,IYDIAG,NRY,NCY,
     *                                   Z,IZZERO,IZTRNS,IZDIAG,NRZ,NCZ)
 
C     THIS SUBROUTINE CALCULATES THE FOLLOWING MATRIX PRODUCTS
C                  CASE 1    X = Y *ZD
C                  CASE 2    X = YD*Z
C                  CASE 3    X = YD*ZD
C                  CASE 4    X = YD*ZT
C                  CASE 5    X = YT*ZD
C                  CASE 6    X = Y *Z
C                  CASE 7    X = YT*Z
C                  CASE 8    X = Y *ZT
C                  CASE 9    X = YT*ZT
C     THE ARGUMENTS USED HAVE THE FOLLOWING DEFINITIONS
C             X      -   INDICATES THE MATRIX X
C             IXZERO = 0 INDICATES THE MATRIX X IS ZERO
C             NRX    -   INDICATES THE NUMBER OF ROWS OF X
C             NCX    -   INDICATES THE NUMBER OF COLUMNS OF X
C             Y      -   INDICATES THE MATRIX Y
C             IYZERO = 0 INDICATES THE MATRIX Y IS ZERO
C             IYTRNS = 1 INDICATES THE MULT INVOLVES Y TRANSPOSE
C             IYDIAG = 1 INDICATES THE MATRIX Y IS A DIAGONAL MATRIX
C             NRY    -   INDICATES THE NUMBER OF ROWS OF Y
C             NCY    -   INDICATES THE NUMBER OF COLUMNS OF Y
C             Z      -   INDICATES THE MATRIX Z
C             IZZERO = 0 INDICATES THE MATRIX Z IS ZERO
C             IZTRNS = 1 INDICATES THE MULT INVOLVES Z TRANSPOSE
C             IZDIAG = 1 INDICATES THE MATRIX Z IS A DIAGONAL MATRIX
C             NRZ    -   INDICATES THE NUMBER OF ROWS OF Z
C             NCZ    -   INDICATES THE NUMBER OF COLUMNS OF Z
 
      DOUBLE PRECISION X,Y,Z
 
      common /tapes/   nouttp
 
      DIMENSION        X(  1,  1),Y(  1,  1),Z(  1,  1)
 
      IYT = IYTRNS
      IYD = IYDIAG
      IZT = IZTRNS
      IZD = IZDIAG
 
C     IF EITHER MATRIX Y OR Z IS ZERO, THEN SET IXZERO TO ZERO & RETURN
 
      IF(IYZERO.EQ.0.OR.IZZERO.EQ.0) GO TO 1000
      IXZERO=1
 
C     TEST TO SEE WHICH CASE IS REQUIRED
 
      IF(IYT.EQ.0.AND.IYD.EQ.0.AND.IZT.EQ.0.AND.IZD.EQ.1) GO TO 1
      IF(IYT.EQ.0.AND.IYD.EQ.1.AND.IZT.EQ.0.AND.IZD.EQ.0) GO TO 2
      IF(IYT.EQ.0.AND.IYD.EQ.1.AND.IZT.EQ.0.AND.IZD.EQ.1) GO TO 3
      IF(IYT.EQ.0.AND.IYD.EQ.1.AND.IZT.EQ.1.AND.IZD.EQ.0) GO TO 4
      IF(IYT.EQ.1.AND.IYD.EQ.0.AND.IZT.EQ.0.AND.IZD.EQ.1) GO TO 5
      IF(IYT.EQ.0.AND.IYD.EQ.0.AND.IZT.EQ.0.AND.IZD.EQ.0) GO TO 6
      IF(IYT.EQ.1.AND.IYD.EQ.0.AND.IZT.EQ.0.AND.IZD.EQ.0) GO TO 7
      IF(IYT.EQ.0.AND.IYD.EQ.0.AND.IZT.EQ.1.AND.IZD.EQ.0) GO TO 8
      IF(IYT.EQ.1.AND.IYD.EQ.0.AND.IZT.EQ.1.AND.IZD.EQ.0) GO TO 9
 
C     WRITE AN ERROR MESSAGE IF THIS POINT REACHED
 
      write (nouttp,13)
   13 FORMAT (1X,' ILLEGAL CALL TO SUBROUTINE MXMULT...CHECK THE FOUR ',
     * 'ARGUMENTS IYTRNS,IYDIAG,IZTRNS,IZDIAG IN THE CALLING SEQUENCE ')
      STOP 'MXMULT: illegal call to this subroutine; abnormal end'
 
C     CASE 1       X = Y *ZD
 
    1 NRX = NRY
      NCX = NCZ
      CALL YY31(X,MRX,Y,NRY,Z,NRZ,NCZ)
      GO TO 2000
 
C     CASE 2       X = YD*Z
 
    2 NRX = NRY
      NCX = NCZ
      CALL YY32(X,MRX,Y,NRY,Z,NRZ,NCZ)
      GO TO 2000
 
C     CASE 3       X = YD*ZD
 
    3 NRX = NRY
      NCX = NCZ
      CALL YY33(X,MRX,Y,NRY,Z,NRZ,NCZ)
      GO TO 2000
 
C     CASE 4       X = YD*ZT
 
    4 NRX = NRY
      NCX = NRZ
      CALL YY34(X,MRX,Y,NRY,Z,NRZ)
      GO TO 2000
 
C     CASE 5       X = YT*ZD
 
    5 NRX = NCY
      NCX = NCZ
      CALL YY35(X,MRX,Y,NRY,Z,NRZ,NCZ,NCY)
      GO TO 2000
 
C     CASE 6       X = Y *Z
 
    6 NRX = NRY
      NCX = NCZ
      CALL YY36(X,MRX,Y,NRY,Z,NRZ,NCZ,NCY)
      GO TO 2000
 
C     CASE 7       X = YT*Z
 
    7 NRX = NCY
      NCX = NCZ
      CALL YY37(X,MRX,Y,NRY,Z,NRZ,NCZ,NCY)
      GO TO 2000
 
C     CASE 8       X = Y *ZT
 
    8 NRX = NRY
      NCX = NRZ
      CALL YY38(X,MRX,Y,NRY,Z,NRZ,NCY)
      GO TO 2000
 
C     CASE 9       X = YT*ZT
 
    9 NRX = NCY
      NCX = NRZ
      CALL YY39(X,MRX,Y,NRY,Z,NRZ,NCY)
      GO TO 2000
 1000 IXZERO=0
      IF (IYT.EQ.1  .AND.  IZT.EQ.0 )  GO TO 1110
      IF ( IYT.EQ.0 .AND. IZT.EQ.1 ) GO TO 1120
      IF ( IYT.EQ.1 .AND.  IZT.EQ.1 ) GO TO 1130
 
C     IYT=0,IZT=0
 
      NRX=NRY
      NCX=NCZ
      GO TO 1200
 1110 NRX=NCY
      NCX=NCZ
      GO TO 1200
 1120 NRX=NRY
      NCX=NRZ
      GO TO 1200
 1130 NRX=NCY
      NCX=NRZ
 1200 IF (NRX.EQ.0 .OR. NCX.EQ.0 )  GO TO 2000
      CALL YY40(X,MRX,NRX,NCX)
 2000 CONTINUE
      RETURN
 
      END
C
      SUBROUTINE YY31(X,MRX,Y,NRY,Z,NRZ,NCZ)
      DOUBLE PRECISION X,Y,Z
      DIMENSION        X(MRX,  1),Y(NRY,  1),Z(NRZ,  1)
      DO 10 I = 1,NRY
      DO 10 J = 1,NCZ
   10 X(I,J) = Y(I,J)*Z(J,1)
      RETURN
      END
 
      SUBROUTINE YY32(X,MRX,Y,NRY,Z,NRZ,NCZ)
      DOUBLE PRECISION X,Y,Z
      DIMENSION        X(MRX,  1),Y(NRY,  1),Z(NRZ,  1)
      DO 20 I = 1,NRY
      DO 20 J = 1,NCZ
   20 X(I,J) = Y(I,1)*Z(I,J)
      RETURN
      END
 
      SUBROUTINE YY33(X,MRX,Y,NRY,Z,NRZ,NCZ)
      DOUBLE PRECISION X,Y,Z
      DIMENSION        X(MRX,  1),Y(NRY,  1),Z(NRZ,  1)
      DO 30 I = 1,NRY
      DO 30 J = 1,NCZ
      X(I,J) = 0.0D0
   30 X(I,I) = Y(I,1)*Z(I,1)
      RETURN
      END
 
      SUBROUTINE YY34(X,MRX,Y,NRY,Z,NRZ)
      DOUBLE PRECISION  X,Y,Z
      DIMENSION         X(MRX,  1),Y(NRY,  1),Z(NRZ,  1)
      DO 40 I = 1,NRY
      DO 40 J = 1,NRZ
   40 X(I,J) = Y(I,1)*Z(J,I)
      RETURN
      END
 
      SUBROUTINE YY35(X,MRX,Y,NRY,Z,NRZ,NCZ,NCY)
      DOUBLE PRECISION  X,Y,Z
      DIMENSION         X(MRX,  1),Y(NRY,  1),Z(NRZ,  1)
      DO 50 I = 1,NCY
      DO 50 J=1,NCZ
   50 X(I,J) = Y(J,I)*Z(J,1)
      RETURN
      END
 
      SUBROUTINE YY36(X,MRX,Y,NRY,Z,NRZ,NCZ,NCY)
      DOUBLE PRECISION  X,Y,Z,SUM
      DIMENSION         X(MRX,  1),Y(NRY,  1),Z(NRZ,  1)
      DO 60 I = 1,NRY
      DO 60 J = 1,NCZ
      SUM  = 0.0D0
      DO 61 K = 1,NCY
   61 SUM = SUM + Y(I,K)*Z(K,J)
   60 X(I,J) = SUM
      RETURN
      END
 
      SUBROUTINE YY37(X,MRX,Y,NRY,Z,NRZ,NCZ,NCY)
      DOUBLE PRECISION X,Y,Z,SUM
      DIMENSION        X(MRX,  1),Y(NRY,  1),Z(NRZ,  1)
      DO 70 I = 1,NCY
      DO 70 J = 1,NCZ
      SUM  = 0.0D0
      DO 71 K = 1,NRY
   71 SUM  = SUM + Y(K,I)*Z(K,J)
   70 X(I,J) = SUM
      RETURN
      END
 
      SUBROUTINE YY38(X,MRX,Y,NRY,Z,NRZ,NCY)
      DOUBLE PRECISION X,Y,Z,SUM
      DIMENSION        X(MRX,  1),Y(NRY,  1),Z(NRZ,  1)
      DO 80 I = 1,NRY
      DO 80 J = 1,NRZ
      SUM  = 0.0D0
      DO 81 K = 1,NCY
   81 SUM  = SUM + Y(I,K)*Z(J,K)
   80 X(I,J) = SUM
      RETURN
      END
 
      SUBROUTINE YY39(X,MRX,Y,NRY,Z,NRZ,NCY)
      DOUBLE PRECISION  X,Y,Z,SUM
      DIMENSION         X(MRX,  1),Y(NRY,  1),Z(NRZ,  1)
      DO 90 I = 1,NCY
      DO 90 J = 1,NRZ
      SUM  = 0.0D0
      DO 91 K = 1,NRY
   91 SUM  = SUM + Y(K,I)*Z(J,K)
   90 X(I,J) = SUM
      RETURN
      END
 
      SUBROUTINE YY40(X,MRX,NRX,NCX)
      DOUBLE PRECISION X
      DIMENSION        X(MRX,  1)
      DO 1250 I=1,NRX
      DO 1250 J=1,NCX
 1250 X(I,J)=0.D0
      RETURN
      END
