CSECT   OPFNGR
C       2 MAY 73   (WNR)
 
      SUBROUTINE OPFNGR (N,G,F,X,M1,KWAY)
 
C     SUBROUTINE TO CALCULATE OPTIMIZATION OBJECTIVE FUNCTION AND
C     ITS GRADIENT
C     N = NUMBER OF INDEPENDENT VARIABLES
C     X = VECTOR OF INDEPENDENT VARIABLES VALUES
C     F = OBJECTIVE FUNCTION VALUE
C     G = VECTOR OF VALUES OF GRADIENT OF OBJECTIVE FUNCTION
C     M1= 1, WHEN OPFNGR IS CALLED THE FIRST TIME BY THE OPTIMIZATION
C            PROCEDURE;
C       = 2, FOR ALL SUBSEQUENT CALLS TO OPFNGR, EXCEPT
C       = 3, FOR THE FINAL CALL.
C     KWAY = DC SOLUTION ERROR SWITCH
 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z),INTEGER (I-N)
 
      COMMON /CNTRLS/ CT(160)
      COMMON /NAMES/  E(1)
      COMMON /VOLTGS/ V(1)
      COMMON /CURNTS/ C(1)
      COMMON /PRMTRS/ P(1)
      COMMON /IVBNDS/ B(3,1)
      COMMON /OPTIMB/ DV(6,1)
      COMMON /OPTIMC/ IV(6,1)
      COMMON /GRADNT/ GD(1)
      COMMON /OPDVCM/ IS1(8),SK1(22),D,IS3,IDV,K,ISW1,ISW2,IVSET
      common /tapes/  nout
      INTEGER         DV,DVNDX,DVTYP
      DIMENSION       X(*),G(*)
 
      DATA XLB,XUB   /-3.14159265358979D0,3.14159265358979D0/
 
      XFM(R,S,T)=R-S*COS(T)
 
C     SUPPRESS DC PRINTOUT.
 
      CT(115)=1.D0
 
C     IF FIRST ENTRY, SKIP DC SOLUTION, WHICH HAS ALREADY TAKEN PLACE.
 
      IF (M1-2) 60,10,150
 
C     STORE VALUES OF INDEPENDENT VARIABLES IN THE PROPER LOCATIONS
C     IN THE APPROPRIATE COMMON.
 
   10 LC=IVSET-1
      DO 30 I=1,N
      LD=LC+I
      L=IV(5,LD)
      IF (IV(6,LD).EQ.4) GO TO 20
      E(L)=XFM(B(2,I),B(3,I),X(I))
      GO TO 30
   20 P(L)=XFM(B(2,I),B(3,I),X(I))
   30 CONTINUE
 
C     CALCULATE DC SOLUTION
 
      CALL SIMDC (KWAY)
 
C     CHECK FOR DC SOLUTION ERROR.
 
      IF (KWAY.NE.0) GO TO 130
 
C     CHECK IF TRANSFORMED INDEPENDENT VARIABLE VECTOR VALUES ARE
C     WITHIN BOUNDS.
 
      DO 55 I=1,N
      IF (X(I).GT.XLB.AND.X(I).LT.XUB) GO TO 55
 
C     A TRANSFORMED INDEPENDENT VARIABLE IS OUT OF BOUNDS.  SET F TO
C     TWICE THE VALUE OF THE DEPENDENT VARIABLE SAVED FROM THE LAST
C     PRECEDING CALL TO OPFNGR AND CONTINUE.
 
      F=SVF+SVF
      GO TO 110
   55 CONTINUE
 
C     LOCATE AND SET DEPENDENT VARIABLE INTO F.
 
   60 DVNDX=DV(1,IDV)
      DVTYP=DV(2,IDV)
      GO TO (70,80,90,100),DVTYP
   70 F=V(DVNDX)
      GO TO 110
   80 F=C(DVNDX)
      GO TO 110
   90 F=E(DVNDX)
      GO TO 110
  100 F=P(DVNDX)
 
C     SAVE DEPENDENT VARIABLE VALUE.
 
  110 SVF=F
 
C     CALL ADJOINT ROUTINE TO CALCULATE GRADIENT.  ALL ELEMENT,
C     VOLTAGE, CURRENT AND DEFINED PARAMETER VALUES WILL BE SAVED
C     AND THEN RESTORED.
 
      CT(144)=0.D0
      CALL ADJNT (DV(3,IDV),N,IV(1,IVSET),KWAY)
 
C     CHECK FOR ADJOINT SOLUTION ERROR.
 
      IF (KWAY.NE.0) GO TO 130
 
C     MOVE GRADIENT VALUES INTO G VECTOR.
 
      DO 120 I=1,N
  120 G(I)=B(3,I)*SIN(X(I))*GD(I)
      GO TO 150
 
C     DC OR ADJOINT SOLUTION FAILURE.  WRITE ERROR MESSAGE.
 
  130 WRITE (NOUT,140)
  140 FORMAT (41H0DC SOLUTION FAILURE. (CALL FROM OPFNGR.)//)
 
C     RESTORE DC PRINTOUT.
 
  150 CT(115)=0.D0
      RETURN
 
      END
