CSECT   PRDCOR                          72.110  00.39.08
 
      SUBROUTINE PRDCOR
     *     (CURNS,NCL3,VOLTS,NCL4,P,NDPD,PSAVE,NROW,A,IGO)
 
C  THE PARAMETERS TO THE SUBROUTINE PRDCOR HAVE THE FOLLOWING
C  MEANINGS:
C
C  CURNS     STORAGE FOR THE INDUCTIVE STATE VARIABLE VALUES.
C  NCL3      THE NUMBER OF INDUCTIVE STATE VARIABLES
C  VOLTS     STORAGE FOR THE CAPACITIVE STATE VARIABLE VALUES.
C  NCL4      THE NUMBER OF CAPACITIVE STATE VARIABLES.
C  P         STORAGE FOR THE DEFINED PARAMETER DERIVATIVE VALUES.
C  NDPD      THE NUMBER OF DEFINED PARAMETER DERIVATIVES
C  PSAVE     STORAGE FOR THE PARTIAL DERIVATIVES.
C  NROW      THE DIMENSION OF PSAVE
C  A         THE COEFFICIENTS THAT DETERMINE THE ORDER.
C  IGO       A 'GO TO' SWITCH WHOSE VALUE IS DETERMINED IN THIS
C              ROUTINE AND USED IN THE GEAR ROUTINE.
 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
 
      INTEGER*2       KQDAT,NDXTAB
      INTEGER*2       LOCA,LCHAIN
      COMMON /GEARSP/ NPVT,NZA,NZC,LOCA(1)
         DIMENSION    IA(1)
         EQUIVALENCE (IA(1),LOCA(1))
      COMMON /GSPCHN/ LCHAIN(1)
      COMMON /VALUES/ VALUE(1)
      COMMON /UTILTY/ Y
      COMMON /GEARTA/ SAVE
      COMMON /CNTRLS/ CTRL(70)
         EQUIVALENCE (T,  CTRL(1)),(CTRL(6),  HMIN),(CTRL(7), HMAX),
     *               (EPS,CTRL(8)),(CTRL(31),XPASS),(CTRL(30),XSTEP),
     *               (XDIFF,CTRL(61))
      COMMON /GEAREQ/ NDXTAB(30),KQDAT(1)
      COMMON /GEARDT/ BND,H,N,NQ,IWEVAL,N7,K
      COMMON /DERIVS/ DERIV(1)
      COMMON /GEARTB/ IP(3)
      DIMENSION       VOLTS(*),CURNS(*),P(*)
      DIMENSION       Y(8,2),SAVE(12,1)
      DIMENSION       A(7,6),PSAVE(NROW,NROW)
 
C  THIS SECTION COMPUTES THE PREDICTED VALUES BY EFFECTIVELY
C  MULTIPLYING THE SAVED INFORMATION BY THE PASCAL TRIANGLE
C  MATRIX.
 
  250 T = T + H
      NGO = nint(CTRL(54))
      CTRL(33)=H
      DO 260 J=2,K
         DO 260 J1=J,K
            J2=K-J1+J-1
            DO 260 I=1,N
  260 Y(J2,I)=Y(J2,I)+Y(J2+1,I)
 
C  UP TO 3 CORRECTOR ITERATIONS ARE TAKEN.  CONVERGENCE IS TESTED
C  BY REQUIRING CHANGES TO BE LESS THAN BND WHICH IS DEPENDENT ON
C  THE ERROR TEST CONSTANT.
C   THE SUM OF THE CORRECTIONS IS ACCUMULATED IN THE ARRAY
C  SAVE(11,I).  IT IS EQUAL TO THE K-TH DERIVATIVE OF Y MULTIPLIED
C  BY H**K/(FACTORIAL(K-1)*A(K,N7)), AND IS THEREFORE PROPORTIONAL
C  TO THE ACTUAL ERRORS TO THE LOWEST POWER OF H PRESENT. (H**K)
 
      DO 270 I=1,N
  270 SAVE(11,I)=0.D0
      DO 430 L=1,3
         CALL DIFFUN (CURNS,NCL3,VOLTS,NCL4,P,NDPD,1)
 
C  IF THERE HAS BEEN A CHANGE OF ORDER OR THERE HAS BEEN TROUBLE
C  WITH CONVERGENCE, PSAVE IS RE-EVALUATED PRIOR TO STARTING THE
C  CORRECTOR ITERATION IN THE CASE OF STIFF METHODS.  IWEVAL IS
C  THEN SET TO -1 AS AN INDICATOR THAT IT HAS BEEN DONE.
 
         IF (IWEVAL.LT.1) GO TO 370
         GO TO (275,300),NGO
  275    DO 280 I=1,N
            DO 280 J=1,N
  280    PSAVE(I,J)=0.0d0
         IF (nint(XDIFF).NE.1) GO TO 289
         DO 281 I=1,N
            VALUE(I)=DERIV(I)
  281    SAVE(9,I)=Y(1,I)
         DO 283 J=1,N
            R=EPS*MAX(EPS,ABS(SAVE(9,J)))
            Y(1,J)=Y(1,J)+R
            D=A(1,N7)*H/R
            CALL DIFFUN (CURNS,NCL3,VOLTS,NCL4,P,NDPD,1)
            DO 282 I=1,N
  282       PSAVE(I,J)=(DERIV(I)-VALUE(I))*D
  283    Y(1,J)=SAVE(9,J)
         DO 284 I=1,N
            PSAVE(I,I)=PSAVE(I,I)+1.D0
  284    DERIV(I)=VALUE(I)
         GO TO 295

  289    CALL PDRV1
         CALL DOLPEQ (NDXTAB,KQDAT,PSAVE,N,N)
         CALL PDERIV (PSAVE)
         R=A(1,N7)*H
         DO 292 I=1,N
            DO 290 J=1,N
  290       PSAVE(I,J)=PSAVE(I,J)*R
  292    PSAVE(I,I)=PSAVE(I,I)+1D0
  295    IWEVAL =-1
         CALL LINSOL (4HPSAV,PSAVE,VALUE,IP,N,1,IERR)
         GO TO 370

  300    DO 305 I=1,NZA
  305    PSAVE(I,1)=0.D0
         CALL PDRV1
         CALL DOLPEQ (NDXTAB,KQDAT,PSAVE,N,1)
         N1=4*N
         IF (nint(XPASS).LE.1) GO TO 320
 
C      RESET IA AND IP VECTORS
 
         DO 315 I=1,N
  315    IA(N +I)=I
  320    R=A(1,N7)*H
         DO 325 I=1,N
            ISTROW=IA(I)+N1
            IEDROW=NZA+N1
            IF (I.LT.N) IEDROW=IA(I+1)+N1-1
            DO 325 J=ISTROW,IEDROW
               M=J-N1
               PSAVE(M,1)=PSAVE(M,1)*R
               ICOL=LOCA(J)
               IF (ABS(ICOL).EQ.I) PSAVE(M,1)=PSAVE(M,1)+1.D0
  325    CONTINUE
         ICNTL=4
         IF (nint(XPASS).LE.1) ICNTL=3
         IZC=NZC
         CALL SOLVE(KWAY,N,NZA,IZC,IA(1),IA(N +1),LOCA(N1+1),PSAVE,
     *           VALUE,ICNTL,NPVT,LCHAIN)
         IF (NPVT.EQ.1) GO TO 300
         IWEVAL=-1
  370    DO 380 I=1,N
  380    VALUE(I)  =Y(2,I)-DERIV(I)*H
         GO TO (385,405),NGO
  385    CALL LINSOL (4HPSAV,PSAVE,VALUE,IP,N,2,IERR)
         GO TO 415
  405    CALL SOLVE (KWAY,N,NZA,IZC,IA(1),IA(N +1),LOCA(N1+1),PSAVE,
     *           VALUE,5,NPVT,LCHAIN)
  415    NT=N
         DO 420 I=1,N
            Y(1,I)=Y(1,I)+A(1,N7)*VALUE(I)
            Y(2,I)=Y(2,I)-VALUE(I)
            SAVE(11,I)=SAVE(11,I)+VALUE(I)
            IF (ABS(VALUE(I)).LE.(BND*SAVE(12,I))) NT=NT-1
  420    CONTINUE
         XPASS=XPASS+1d0
         IF (NT.LE.0) GO TO 490
  430 CONTINUE

      IGO = 1
      RETURN

  490 IGO=2
      RETURN
 
      END
