CSECT   SENSI
C       16 APR 73
 
      SUBROUTINE SENSI (KWAY)
 
C             ..... SETS UP ALL SENSITIVITY CALCULATIONS
C
C     NSETS=NUMBER OF SENSITIVITY SETS (OF DEPENDENT VARIABLES)
C
C     KSETS(4,NSETS)- SENSITIVITY SET TABLE
C     KSETS(1,I)=INDEX TO DEPENDENT VARIABLE TABLE 'KDEP'
C     KSETS(2,I)=NUMBER OF DEPENDENT VARIABLES FOR SET 'I'
C     KSETS(3,I)=INDEX TO INDEPENDENT VARIABLE TABLE 'KIND'
C     KSETS(4,I)=NUMBER OF INDEPENDENT VARIABLES FOR SET 'I'
C
C     KDEP(4,NDEP)-DEPENDENT VARIABLE TABLE
C     KDEP(1,I)=INDEX TO DEPENDENT VARIABLE NAME IN NAMES TABLE
C     KDEP(2,I)=FUNCTION TYPE (1=VOLTAGE,2=CURRENT,4=DEFINED PARAMETER)
C     KDEP(3,I)=INDEX TO LIST OF ADJOINT SOURCES 'KASRC'
C     KDEP(4,I)=LENGTH OF ADJOINT SOURCE LIST FOR DEP. VAR. 'I'
C     KDEP(5,I)=INDEX TO LIST OF ADDITIVE TERMS FROM DIFFERENTIAL
C     KDEP(6,I)=LENGTH OF LIST OF ADDITIVE TERMS
C
C     KIND(4,NIND)-INDEPENDENT VARIABLE TABLE
C     KIND(1,I)=PLACE TO MONITOR IN INITIAL CONDITIONS CALCULATION
C     KIND(2,I)=SYSTEM FUNCTION (1=VOLT,2=CURRENT,3=VALUE)
C     KIND(3,I)=PLACE TO MONITOR IN ADJOINT CALCULATION
C     KIND(4,I)=SYSTEM FUNCTION (1=VOLT,2=CURRENT,3=VALUE)
C     KIND(5,I)=INDEX TO INDEP. VARIABLE IN ELEMENTS OF DEF. PAR. TABLES
C     KIND(6,I)=TYPE (3=ELEMENT, 4=DEFINED PARAMETER)
 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER (I-N)
 
      CHARACTER       SET*4,BLANK*8,SYSFN*1,SFN(4)*1,DOT*1,D*1
      COMMON /ADJDAT/ NIS,NSETS
      COMMON /CNTRLS/ CTRLS(144)
         EQUIVALENCE (CTRLS(115),XICPRT),(CTRLS(116),XICDBG)
         EQUIVALENCE (CTRLS(143),XSTORD),(CTRLS(144),XRESET)
      COMMON /DPNAMS/ DPS(1)
      COMMON /EBACK/  EHOLD(3,1)
      COMMON /ELNAMS/ ELNAMS(1)
      COMMON /GRADNT/ GRDNT(1)
      COMMON /PBACK/  PHOLD(1)
      COMMON /SENSA/  KSETS(4,1)
      COMMON /SENSB/  KDEP(6,1)
      COMMON /SENSC/  KIND(6,1)
      COMMON /TAPES/  NOUTTP
      CHARACTER*8     DPS,ELNAMS,DEPVAR,AIVVAR
 
      DATA BLANK /' '/
      DATA DOT   /'.'/
      DATA SFN  /'V','I',' ',' '/
 
      WRITE (NOUTTP,5000)
 
C     TURN RESET INDICATOR OFF SO 'ADJNT' WILL NOT RESET
 
      XRESET=1.d0
 
C     TURN IC PRINTING OFF, IF REQUESTED
 
      IF (nint(XICDBG).EQ.0) XICPRT=1.D0
      DO 1000 I=1,NSETS
         D=DOT
         write (set,'(i4)') i
 
C     IF LAST SET, TELL 'ADJNT' TO RESET VALUES
 
         IF (I.GE.NSETS) XRESET=0.d0
         IDEP=KSETS(1,I)
         NDEP=KSETS(2,I)
         ISTOP=IDEP+NDEP-1
         INDPV=KSETS(3,I)
         NINDV=KSETS(4,I)
         DO 900 IDX=IDEP,ISTOP
            KWAY=0
            CALL ADJNT (KDEP(3,IDX),NINDV,KIND(1,INDPV),KWAY)
            IX=KDEP(1,IDX)
            IY=KDEP(2,IDX)
            IF (IY.EQ.4) then
               DEPVAR=DPS(IX)
               SDNOM=PHOLD(IX)
             else
               DEPVAR=ELNAMS(IX)
               SDNOM=EHOLD(IY,IX)
            end if
            SYSFN=SFN(IY)
            IF (KWAY.ne.0) then
               WRITE (NOUTTP,5100) SYSFN,DEPVAR,KWAY
               GO TO 1000
            end if
            IZZ=INDPV-1
            DO 150 IG=1,NINDV
               IZZ=IZZ+1
               IZ=KIND(5,IZZ)
               JZ=KIND(6,IZZ)
               IF (JZ.EQ.4) then
                  AIVVAR=DPS(IZ)
                  SNUM=PHOLD(IZ)
                else
                  AIVVAR=ELNAMS(IZ)
                  SNUM=EHOLD(3,IZ)
               end if
               IF (ABS(SDNOM).ge.1.D-20) then
                  SENS=GRDNT(IG)*SNUM/SDNOM
                  WRITE (NOUTTP,5050) SET,D,SYSFN,DEPVAR,AIVVAR,
     *                  GRDNT(IG),SENS
                else
 
C     ZERO DIVIDE, UNDEFINED SENSITIVITY
 
                  WRITE (NOUTTP,5060) SET,D,SYSFN,DEPVAR,AIVVAR,
     *                  GRDNT(IG)
               end if
               D=BLANK
               SET=BLANK
               SYSFN=BLANK
               DEPVAR=BLANK
  150       CONTINUE
            WRITE (NOUTTP,5200)
  900    CONTINUE
         WRITE (NOUTTP,5200)
 1000 CONTINUE
 
C     TURN IC PRINTING ON
 
      XICPRT=0.D0
      RETURN
 
 5000 FORMAT('1__________________________'/
     *       '+SENSITIVITY CALCULATION(S)'/
     *       '0 SET',7X,'DEPENDENT',2X,'INDEPENDENT',6X,'PARTIAL',12X,
     *       'NORMALIZED'/
     *       ' NUMBER',5X,'VARIABLE',4X,'VARIABLE',6X,'DERIVATIVE',11X,
     *       'SENSITIVITY'//)
 5050 FORMAT(' ',A4,A1,7X,A1,A5,6X,A6,5X,1P,E15.8,6X,E15.8)
 5060 FORMAT(' ',A4,A1,7X,A1,A5,6X,A6,5X,1P,E15.8,7X,'*** UNDEFINED')
 5100 FORMAT('0ADJOINT IC SOLUTION FAILURE ON DEPENDENT VARIABLE ',A1,
     *       A8,', KWAY = ',I4//)
 5200 FORMAT(' ')
 
      END
