CSECT   SKALEI
 
      SUBROUTINE SKALEI(N,NM,A,H,PRFACT,ENORM)
 
C THIS SUBROUTINE STORES THE MATRIX OF THE ORDER N FROM THE
C ARRAY A INTO THE ARRAY H. AFTERWARD THE MATRIX IN THE
C ARRAY A IS SCALED SO THAT THE QUOTIENT OF THE ABSOLUTE SUM
C OF THE OFF-DIAGONAL ELEMENTS OF COLUMN I AND THE ABSOLUTE
C SUM OF THE OFF-DIAGONAL ELEMENTS OF ROW I LIES WITHIN THE
C VALUES OF BOUND1 AND BOUND2.
C THE COMPONENT I OF THE EIGENVECTOR OBTAINED BY USING THE
C SCALED MATRIX MUST BE DIVIDED BY THE VALUE FOUND IN THE
C PRFACT(I) OF THE ARRAY PRFACT. IN THIS WAY THE EIGENVECTOR
C OF THE NON-SCALED MATRIX IS OBTAINED.
C
C AFTER THE MATRIX IS SCALED IT IS NORMALISED SO THAT THE
C VALUE OF THE EUCLIDIAN NORM IS EQUAL TO ONE.
C IF THE PROCESS OF SCALING WAS NOT SUCCESSFUL THE ORIGINAL
C MATRIX FROM THE ARRAY H WOULD BE STORED BACK INTO A AND
C THE EIGENPROBLEM WOULD BE SOLVED BY USING THIS MATRIX.
C NM DEFINES THE FIRST DIMENSION OF THE ARRAYS A AND H. NM
C MUST BE GREATER OR EQUAL TO N.
C THE EIGENVALUES OF THE NORMALISED MATRIX MUST BE
C MULTIPLIED BY THE SCALAR ENORM IN ORDER THAT THEY BECOME
C THE EIGENVALUES OF THE NON-NORMALISED MATRIX.
 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER (I-N)
 
      DIMENSION A(NM,1),H(NM,1),PRFACT(NM)
 
      DO 2 I=1,N
      DO 1 J=1,N
    1 H(I,J) = A(I,J)
    2 PRFACT(I) = 1.0D0
      BOUND1 = 0.75D0
      BOUND2 = 1.33D0
      ITER = 0
    3 NCOUNT = 0
      DO 8 I=1,N
      COLUMN = 0.0D0
      ROW = 0.0D0
      DO 4 J=1,N
      IF(I.EQ.J) GO TO 4
      COLUMN = COLUMN+DABS(A(J,I))
      ROW = ROW+DABS(A(I,J))
    4 CONTINUE
      IF(COLUMN.EQ.0.0D0) GO TO 5
      IF(ROW.EQ.0.0D0) GO TO 5
      Q = COLUMN/ROW
      IF(Q.LT.BOUND1) GO TO 6
      IF(Q.GT.BOUND2) GO TO 6
    5 NCOUNT = NCOUNT+1
      GO TO 8
    6 FACTOR = DSQRT(Q)
      DO 7 J=1,N
      IF(I.EQ.J) GO TO 7
      A(I,J) = A(I,J)*FACTOR
      A(J,I) = A(J,I)/FACTOR
    7 CONTINUE
      PRFACT(I) = PRFACT(I)*FACTOR
    8 CONTINUE
      ITER = ITER+1
      IF(ITER.GT.30) GO TO 11
      IF(NCOUNT.LT.N) GO TO 3
 
      FNORM = 0.0D0
      DO 9 I=1,N
      DO 9 J=1,N
      Q = A(I,J)
    9 FNORM = FNORM+Q*Q
      FNORM = DSQRT(FNORM)
      DO 10 I=1,N
      DO 10 J=1,N
   10 A(I,J) = A(I,J)/FNORM
      ENORM = FNORM
      GO TO 13
 
   11 DO 12 I=1,N
      PRFACT(I) = 1.0D0
      DO 12 J=1,N
   12 A(I,J) = H(I,J)
      ENORM = 1.0D0
 
   13 RETURN
 
      END
