CSECT   SOLVE                           72.110  00.39.08
 
      SUBROUTINE SOLVE
     *   (IERROR,N,NZA,NZC,IA,IP,LOCA,C,B,NGOTO,NPIVOT,LCOL)
 
C        THIS SUBROUTINE TAKES ARRAYS IA, IP, AND LOCA AND
C        SOLVES THE LINEAR SYSTEM.
C           AX=B
C        AND C IS A ONE DIMENSIONAL ARRAY OF LENGTH NZC AND THE FIRST
C        NZA ELEMENTS OF C CONTAIN THE NON-ZERO ELEMENTS OF A.  THE
C        SOLUTION VECTOR OF X IS PLACED IN B.
C        THE SOLUTION IS BY TRIANGULARIZATION.  THAT IS MATRIX A IS
C        DECOMPOSED INTO A PRODUCT OF A LOWER TRIANGULAR MATRIX L AND
C        AND AN UPPER TRIANGLE MATRIX U.  SEE NOBLE, BEN, 'APPLIED
C        LINEAR ALGEBRA' (PRELIMINARY EDITION), PAGES 133-138.
C        THIS MATRIX IS TREATED AS A SINGULAR MATRIX (C), AND THE ONLY
C        ELEMENTS RETAINED ARE THE NON-ZERO ELEMENTS.
C        USAGE.
C             CALL SOLVE(IERROR,N,NZA,NZC,IA,IP,LOCA,C,B,NGOTO,
C            *NPIVOT,LCOL)
C        WHERE.
C             IERROR - ERROR INDICATOR.  IT IS ZERO IF SOLUTION WAS
C                      PERFORMED.  IT IS 10 IF THE NEWTON-RAPHSON
C                      PASS LIMIT IS EXCEEDED WITHOUT CONVERGENCE OR
C                      IF THE MATRIX IS SINGULAR.
C             N      - THE ORDER OF MATRIX A.  IT IS ALSO THE DIMENSION
C                      OF ARRAYS B, IA AND IP.
C             NZA    - THE NUMBER OF NON-ZERO ELEMENTS OF A.
C             NZC    - THE DIMENSION OF ARRAYS C, AND LOCA.  IT IS THE
C                      NUMBER OF NON-ZERO ELEMENTS OF THE LOWER
C                      PLUS THE UPPER TRIANGULAR MATRICES.
C             IA     - ARRAY OF LENGTH N, WHICH HAS THE INDICES OF
C                      LOCA WHERE EVERY N ROW STARTS.
C             LOCA   - ARRAY OF LENGTH NZC, WHICH HAS THE LOCATIONS OF
C                      THE NON-ZERO ELEMENTS OF ARRAY C, ORIENTED ROW
C                      WISE.
C             IP     - THE PERMUTATION OF THE ROWS OF A IN LOCA.
C             C      - ARRAY OF LENGTH NZC, WHICH CONTAINS THE ACTUAL
C                      VALUES OF THE NON-ZERO ELEMENTS OF A IN THE
C                      FIRST NZA LOCATIONS.  ON RETURN THEY ARE
C                      DESTROYED.
C             B      - ARRAY OF LENGTH N, WHICH CONTAINS THE VALUES OF
C                      THE RIGHT HAND VECTOR.  ON RETURN IT CONTAINS
C                      THE SOLUTION VECTOR
C             NGOTO - =1, FIRST ENTRY FOR I/C SOLUTION
C                     =2, AFTER FIRST ENTRY FOR I/C SOLUTION
C                     =3, FIRST ENTRY FOR IMPLICIT INTEGRATION,
C                         WILL COMPUTE THE ELEMENTS OF C ONLY
C                     =4, SECOND OR GREATER ENTRY FOR IMPLICIT
C                         INTEGRATION,WILL COMPUTE THE ELEMENTS OF C
C                         ONLY.
C                     =5, CALLED BY IMPLICIT INTEGRATION ROUTINE,
C                         WILL COMPUTE SOLUTION DURING THIS PASS.
C              NPIVOT - =0, PIVOTING IS OKAY
C                       =1, CAN NOT PIVOT BECAUSE OF LACK OF TABLE
C                           SPACE.
C                       =2, CAN NOT PIVOT
C             LCOL  -  A VECTOR OF LENGTH NZC CONTAINING THE INDICES
C                      FOR CHAINING THE ELEMENTS OF EACH ROW OF A.
 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
 
      INTEGER*2       LOCA(2),LCOL(1)
      INTEGER         IA(2),IP(2)
      COMMON /TAPES / NOUTTP,INTAPE,LIBTP,INOUTP,NEDITP,NSAVTP,ISAV
      COMMON /CNTRLS/ TIME  ,XSTOPT,XMXERT,XIR   ,XTISSS,XMNISS,XMXISS,
     *                XMNAIE,XMXAIE,XIC   ,XRERNO,XMXPAS,XMNRIE,XMXRIE,
     *                XMNISO,XMXICP,XICRER,XICAER,XMXOTP,XXDIMS,XYDIMS,
     *                XICPAS,XNOPRQ,XNTNDX,XNHEAD,XNDFEQ,XERT  ,XNRERN,
     *                XCNTNU,XSTPNO,XPASNO,XRUNNO,XSTPSZ
      DIMENSION       C(2),B(2)
 
      DATA ZERO  /0.0d0/
 
   1  IF (NPIVOT.GT.0) NPIVOT=2
      IF (XICPAS.LE.XMXICP) GO TO 4
      WRITE (NOUTTP,3) XMXICP
    3 FORMAT('0THE NEWTON-RAPHSON PASS LIMIT OF ',F5.0,
     *       ' HAS BEEN EXCEEDED WITHOUT ATTAINING CONVERGENCE')
      IERROR=10
      GO TO 467
 
    4 IERROR = 0
      GO TO (5,8,5,8,426) ,NGOTO
 
C     INITIALIZE ON FIRST ENTRY
 
    5 NPIVOT=0
      NGOTO=NGOTO+1
      ICHNGE=0
      GO TO 50
    8 CONTINUE
 
C     GENERATE THE CHAIN FOR THE ELEMENTS IN THE C VECTOR.
 
      IF (ICHNGE.EQ.0) GO TO 90
   50 DO  80 I=1,N
      KSTRT=IA(I)
      KEND=NZA
      IF (I.LT.N) KEND=IA(I+1)-1
   60 LCOL(KSTRT)=KSTRT+1
      IF (KSTRT.GE.KEND) GO TO 70
      KSTRT=KSTRT+1
      GO TO 60
   70 LCOL(KSTRT)=0
   80 CONTINUE
 
C     THE FOLLOWING COMPUTES ALL THE ELEMENTS OF C
 
   90 LIMIT=NZC
      NZC=NZA
      ICHNGE=0
 
C     INITIALIZE THE PERMUTATION VECTOR
 
      DO 100 I=1,N
  100 IP(I)=I
      DO 410 I=1,N
      CMAX=ZERO
      IPOS=0
      IIROW=0
      I2=I-1
      DO 280 I1=I,N
  110 K=IA(IP(I1))
  120 J=LOCA(K)
      IF (J-I2) 130,180,270
  130 IF (LCOL(K).EQ.0) GO TO 315
      K=LCOL(K)
      GO TO 120
  180 L=LSTRT
      JP=K
      IF (LCOL(L).EQ.0) GO TO 265
      GO TO 190
  185 IF (LCOL(L).EQ.0) GO TO 280
  190 L=LCOL(L)
      JJ=LOCA(L)
  195 JPOLD=JP
      JP=LCOL(JP)
      IF (JP.EQ.0) GO TO 260
      IJ =LOCA(JP)
      IF (IJ-JJ) 205,210,260
  205 IF (IJ.NE.I .OR. ABS(C(JP)).LE.CMAX) GO TO 195
      IPOS=JP
      IIROW=I1
      CMAX=ABS(C(JP))
      GO TO 195
  210 C(JP)=C(JP)-C(L)*C(K)
      IF (JJ.NE.I .OR. ABS(C(JP)).LE.CMAX) GO TO 185
      IPOS=JP
      IIROW=I1
      CMAX=ABS(C(JP))
      GO TO 185
 
C     EXPAND LOCA AND C
 
  260 IF (NZC.GE.LIMIT) GO TO 310
      NZC=NZC+1
      LCOL(NZC)=LCOL(JPOLD)
      LCOL(JPOLD)=NZC
      C(NZC)=ZERO
      LOCA(NZC)=JJ
      JP=NZC
      GO TO 210
  265 IF (JP) 280,280,267
  267 K=LCOL(JP)
      J=LOCA(K)
  270 IF (J.NE.I .OR. ABS(C(K)).LE.CMAX) GO TO 280
      IPOS=K
      IIROW=I1
      CMAX=ABS(C(K))
  280 CONTINUE
      IF (IPOS.EQ.0) GO TO 315
      IF (IIROW.EQ.I) GO TO 290
      IF (NPIVOT.GT.0) GO TO 305
 
C     EXCHANGE ROWS
 
      ICHNGE=1
      M3=IP(IIROW)
      IP(IIROW)=IP(I)
      IP(I)=M3
 
C     CALCULATE UPPER MATRIX VALUES FOR PIVOT ROW
 
  290 K=IPOS
  295 IF (LCOL(K).EQ.0) GO TO 410
      DIAG=C(K)
  300 K=LCOL(K)
      C(K)=C(K)/DIAG
      IF (LCOL(K)) 410,410,300
  305 K=IA(IP(I))
      ICHNGE=0
  306 J=LOCA(K)
      IPOS=K
      IF (J-I) 307,295,315
  307 IF (LCOL(K).EQ.0) GO TO 315
      K=LCOL(K)
      GO TO 306

  310 NPIVOT=1
      WRITE (NOUTTP,311)
  311 FORMAT ('0TABLES CONTAINING SPARSE MATRIX DATA HAVE OVERFLOWED.',
     *       ' PIVOTING OF THE MATRIX WILL BE DELETED.'//)
      GO TO 467

  315 IERROR=10
      WRITE (NOUTTP,316)
  316 FORMAT('0SOLVE: THE MATRIX IS SINGULAR.'//)
      GO TO 467

  410 LSTRT=IPOS
 
C             THE FOLLOWING LOOP PERFORMS FORWARD SUBSTITUTION
 
  425 CONTINUE
      IF (NGOTO-3) 426,468,468
  426 DO 440 K=1,N
      I = IP(K)
      L=IA(I)
      NTEMP3=LOCA(L)
      IF (NTEMP3.GE.K) GO TO 440
  430 J=IP(NTEMP3)
      B(I)=B(I)-C(L)*B(J)
      L=LCOL(L)
      NTEMP3=LOCA(L)
      IF (NTEMP3 .LT.K) GO TO 430
  440 B(I)=B(I)/C(L)
 
C             THE FOLLOWING LOOP PERFORMS BACK SUBSTITUTION.
 
      M5=N-1
      DO 450 K=1,M5
      M3 = N - K
      M2 = IP( M3 )
      L = IA(M2)
  443 IF (LOCA(L).GE.M3) GO TO 445
      L=LCOL(L)
      GO TO 443
  445 IF (LCOL(L).EQ.0) GO TO 450
      L=LCOL(L)
      J=IP(LOCA(L))
      B(M2)=B(M2)-C(L)*B(J)
      GO TO 445
  450 CONTINUE
 
C     PUT SOLUTION VECTOR IN CORRECT ORDER
 
      IF (ICHNGE.LE.0 ) GO TO 467
      NNEND=N-1
      DO 462 I=1,NNEND
      IF (IP(I)-I) 451,462,455
  451 NDX=IP(I)
  452 IF (IP(NDX)-I) 453,462,456
  453 NDX=IP(NDX)
      GO TO 452
  455 NDX=I
  456 SAVE=B(I)
      NDX=IP(NDX)
      B(I)=B(NDX)
      B(NDX)=SAVE
      GO TO 462
  462 CONTINUE
  467 IF (NGOTO.LT.5) NZC=LIMIT
  468 CONTINUE
      RETURN
 
      END
