CSECT   TRANIC
 
      SUBROUTINE TRANIC (I3,I3DOT,SVCRNT,NCL3C,V4,V4DOT,SVVOLT,NCL4C,
     *                   PRAMTR,XDPDOT,SVPRAM,NDPD,KWAY)
 
C      THIS SUBROUTINE CONTROLS THE PROCESS OF SOLVING INITIAL
C      CONDITIONS VIA IMPLICIT INTEGRATION.
C
C      ARGUMENTS ARE:
C           I3    - STATE VARIABLE CURRENTS
C           I3DOT - STATE VARIABLE CURRENT DERIVATIVES.
C           SVCRNT- STATE VARIABLE CURRENT NAMES.
C           NCL3C - NUMBER OF STATE VARIABLE CURRENTS.
C           V4    - STATE VARIABLE VOLTAGES.
C           V4DOT - STATE VARIABLE VOLTAGE DERIVATIVES.
C           SVVOLT- STATE VARIABLE VOLTAGE NAMES.
C           NCL4C - NUMBER OF STATE VARIABLE VOLTAGES.
C           PRAMTR- DEFINED PARAMETERS.
C           XDPDOT- DEFINED PARAMETER DERIVATIVES.
C           SVPRAM- DEFINED PARAMETER NAMES.
C           NDPD  - NUMBER OF DEFINED PARAMETER DERIVATIVES.
C           KWAY  - ERROR INDICATOR
C                    =0  DC SOLUTION CONVERGED
C                    =10 DC SOLUTION DID NOT CONVERGE.
 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON /TAPES/  NOUTTP
      COMMON /CNTRLS/ CTRLS(160)
         EQUIVALENCE (CTRLS(1),TIME),(CTRLS(5),XTISSS),
     *               (CTRLS(6),XMNISS),(CTRLS(7),XMXISS),
     *               (CTRLS(12),XMXPAS),(CTRLS(17),XICRER),
     *               (CTRLS(18),XICAER),(CTRLS(30),XSTPNO),
     *               (CTRLS(31),XPASNO),(CTRLS(44),WRTSIM),
     *               (CTRLS(33),XSTPSZ),(CTRLS(115),XICPRT)
      COMMON /BUFFRS/ BUF(7,400)
      REAL            BUF
      DIMENSION       I3(1),I3DOT(1),SVCRNT(1),V4(1),V4DOT(1),
     *                SVVOLT(1),PRAMTR(1),XDPDOT(1),SVPRAM(1),SAVCTR(4)
      DOUBLE PRECISION I3,I3DOT
      CHARACTER*6     STEP
 
      DATA ZERO /0D0/
      DATA STEP /'STEP'/
CIBM: DATA TMAX /Z7FFFFFFFFFFFFFFF/
CVAX: DATA TMAX /ZFFFFFFFFFFFF7FFF/
CFTN77/386      DATA TMAX /Z'7FFFFFFFFFFFFFFF'/
      DATA TMAX /1.d38/
 
C     SAVE CONTROLS USED FOR TRANSIENT SOLUTION.
 
      SAVCTR(1)=XTISSS
      SAVCTR(2)=XMNISS
      SAVCTR(3)=XMXISS
      SAVCTR(4)=XSTPSZ
 
C     INSERT CONTROLS USED FOR DC SOLUTION.
 
      XTISSS=CTRLS(56)
      XMNISS=CTRLS(57)
      XMXISS=CTRLS(58)
      CTRLS(59)=1.D0
      CTRLS(60)=TIME
 
C     PERFORM INTEGRATION
 
      ASSIGN 330 TO JWAY
      ICOL=0
  100 XSTPNO=XSTPNO+1D0
      CALL GEAR (I3,NCL3C,V4,NCL4C,PRAMTR,NDPD)
 
C     SET FLAG IF STATE VARIABLES HAVE NOT CONVERGED TO STEADY STATE VAL
 
      NFLAG=0
      IF (NCL3C.ne.0) then
         DO 130 I=1,NCL3C
            IF (ABS(I3DOT(I)).GT.XICRER*ABS(I3(I))+XICAER) NFLAG=7
  130    CONTINUE
      end if
      IF (NCL4C.ne.0) then
         DO 170 I=1,NCL4C
            IF (ABS(V4DOT(I)).GT.XICRER*ABS(V4(I))+XICAER) NFLAG=7
  170    CONTINUE
      end if
      IF (NDPD.ne.0) then
         DO 230 I=1,NDPD
            IF (ABS(XDPDOT(I)).GT.XICRER*ABS(PRAMTR(I))+XICAER) NFLAG=7
  230    CONTINUE
      end if
 
C     HAS WRITE SIMUL8 DATA BEEN REQUESTED.
 
      IF (nint(WRTSIM).EQ.0) GO TO 330
 
C     IS IC PRINTING REQUESTED
 
      IF (nint(XICPRT).NE.0) GO TO 330
 
C     HAVE TEN INTEGRATION STEPS BEEN TAKEN SINCE STATE
C       VARIABLES VALUES HAVE BEEN SAVED.
 
      IF (MOD(nint(XSTPNO),10).NE.0) GO TO 330
 
C     STORE STATE VARIABLE VALUES INTO BUFFER
 
      NSUM=1
      ICOL=ICOL+1
      BUF(ICOL,1)=real(XSTPNO)
      IF (NCL3C.ne.0) then
         DO 255 I=1,NCL3C
            NSUM=NSUM+1
  255    BUF(ICOL,NSUM)=real(I3(I))
      end if
      IF (NCL4C.EQ.0) then
         DO 265 I=1,NCL4C
            NSUM=NSUM+1
  265    BUF(ICOL,NSUM)=real(V4(I))
      end if
      IF (NDPD.EQ.0) then
         DO 275 I=1,NDPD
            NSUM=NSUM+1
  275    BUF(ICOL,NSUM)=real(PRAMTR(I))
      end if
 
C     IS PRINT BUFFER FULL
 
      IF (ICOL.LT.7) GO TO 330
 
C     PRINT DATA IN BUFFER
 
  285 NSUM=1
      WRITE (NOUTTP,320)
      WRITE (NOUTTP,325) STEP,(BUF(I,1),I=1,ICOL)
      IF (NCL3C.EQ.0) GO TO 295
      DO 290 I=1,NCL3C
      NSUM=NSUM+1
      IF (nint(CTRLS(114)).EQ.0) GO TO 2900
      WRITE (NOUTTP,3250) SVCRNT(I),(BUF(J,NSUM),J=1,ICOL)
 3250 FORMAT(2H I,A5,3X,7(1P,E12.5,2X))
      GO TO 290
 2900 WRITE (NOUTTP,325 ) SVCRNT(I),(BUF(J,NSUM),J=1,ICOL)
  290 CONTINUE
  295 IF (NCL4C.EQ.0) GO TO 305
      DO 300 I=1,NCL4C
      NSUM=NSUM+1
      IF (nint(CTRLS(114)).EQ.0) GO TO 3000
      WRITE (NOUTTP,3260) SVVOLT(I),(BUF(J,NSUM),J=1,ICOL)
 3260 FORMAT(2H V,A5,3X,7(1P,E12.5,2X))
      GO TO 300
 3000 WRITE (NOUTTP,325 ) SVVOLT(I),(BUF(J,NSUM),J=1,ICOL)
  300 CONTINUE
  305 IF (NDPD.EQ.0) GO TO 315
      DO 310 I=1,NDPD
      NSUM=NSUM+1
  310 WRITE(NOUTTP,325) SVPRAM(I),(BUF(J,NSUM),J=1,ICOL)
  315 ICOL=0
 
C     TAKE THE SPECIFIED BRANCH
 
      GO TO JWAY,(330,335,410)
  320 FORMAT('0')
  325 FORMAT(1X,A6,3X,7(1P,E12.5,2X))
 
C     HAVE MAXIMUM INTEGRATION PASSES BEEN EXCEEDED
C     OR IS SMALLER MINIMUM STEP SIZE REQUIRED.
 
  330 IF (XPASNO.LE.XMXPAS.AND.TIME.NE.TMAX) GO TO 400
 
C     IS THERE DATA IN THE PRINT BUFFER
 
      IF (nint(WRTSIM).EQ.0 .OR. ICOL.EQ.0) GO TO 335
      ASSIGN 335 TO JWAY
      GO TO 285
 
C     WRITE ERROR MESSAGE,SET ERROR INDICATOR AND RETURN.
 
  335 IF (XPASNO.GT.XMXPAS) WRITE(NOUTTP,340) nint(XMXPAS)
  340 FORMAT('0TRANIC: THE INTEGRATION PASS LIMIT OF ',i8,
     *       ' HAS BEEN EXCEEDED WITH NO DC STEADY STATE SOLUTION.')
      IF (TIME.EQ.TMAX) WRITE(NOUTTP,345)
  345 FORMAT('0TRANIC: MINIMUM STEP SIZE HAS BEEN EXCEEDED WITH',
     *       ' NO DC STEADY STATE SOLUTION.')
      KWAY=10
      GO TO 410
 
C     IS THERE A DC STEADY STATE SOLUTION?
 
  400 IF (NFLAG.NE.0) GO TO 100
 
C     IS THERE DATA IN THE PRINT BUFFER
 
      IF (nint(WRTSIM).EQ.0 .OR. ICOL.EQ.0) GO TO 410
      ASSIGN 410 TO JWAY
      GO TO 285
 
C     REPLACE TRANSIENT CONTROLS AND RETURN.
 
  410 TIME=CTRLS(60)
      XTISSS=SAVCTR(1)
      XMNISS=SAVCTR(2)
      XMXISS=SAVCTR(3)
      XSTPSZ=SAVCTR(4)
      XPASNO=ZERO
      XSTPNO=ZERO
      CTRLS(59)=ZERO
      RETURN
 
      END
