      SUBROUTINE PARS01 (WINP,WOUT,KTYP,KLEN,ROUT)

      PARAMETER (MAXCOL=72)

C Version: 9.4.86, OCT-1989
C Author : W.-R. Novender
C Remarks: this subroutine examines the input string WINP
C          from its current column position (initially 1).
C          If it finds a specific item (see below), it sets
C          KTYP and KLEN to the appropriate values, copies
C          the item into WOUT(1:KLEN), and converts it into
C          a number, ROUT, if KTYP=2. Each call to PARS01
C          advances the coloumn pointer from its old position
C          up to MAXCOL, then signals the end of line by KLEN=-99.

C This parser distinguishs between keywords, literals, numbers,
C logical values and delimiters.
C
C General delimiters: <BLANK>, <,>, <=>, <EOL> (END OF INPUT LINE)
C Special    "      : <(>, <)>, </>, <$>, <#>, <'>, <.>
C
C The following rules apply to the syntax:
C
C keywords:
C      must begin with a letter, may contain numbers and special
C      characters '-' and '_'.
C      They may not be delimited by <$>, <#>,<'> OR <.>.
C numbers:
C      may only contain digits, the exponent 'D' or 'E', a decimal
C      point and a sign.
C      They may not be delimited by <$>, <#>,<'>, <.> OR <=>.
C logical values:
C      1st character must be a dot, 2nd character1must be 'T' or 'F'.
C      The format should be according to FORTRAN rules.
C      May be delimited only by <BLANK>, <,>, or </>.
C literals:
C      First occurence of a quote <'> starts, 2nd delimits the literal.
C      Within the literal all characters are allowed except the <'>.
C      All delimiters are allowed.
C Note:
C    - neither keywords, logical values, literals nor numbers may
C      span more than 1 input line.
C    - The delimiters <#> and <$> may not directly follow keywords,
C      numbers and logical values.
C    - The delimiter </> may directly follow keywords, numbers, literals
C      and logical value (like the command format in VMS, MS-DOS etc.)
C Examples:
C      PRINT,X1=23,Y2=-1E-10,LEFT=.TRUE.,/SUMMARY=BRIEF
C      X3=1.2 #3 3.4 #10,4,#13,999.
C      AR(12,PY)=OMEGA
C      $NAMELIST A=2 B=-2.3D-5 LPLOT=.F. C(4,7)='TRINIDAD' $END
C Note: the repetition operation <*> within namelists is not supported!

C Exits:
C
C KLEN  =-99  : end of input line
C
C KTYP  = 1   : literal
C KTYP  = 2   : any number
C KTYP  = 3   : special character <#>
C KTYP  = 4   : logical value
C KTYP  = 5   : left parenthesis  <(>
C KTYP  = 6   : right parenthesis <)>
C KTYP  = 7   : special character <$>
C KTYP  = 8   : special character </>
C KTYP  = 9   : key word
C KTYP  =10   : only intermediate type for KTYP=2 or 4
C
C   Errors:
C
C KTYP = -1   : invalid syntax
C KTYP = -2   : invalid number format
C KTYP = -3   : invalid logical format
C KTYP = -4   : invalid keyword syntax
C KTYP = -5   : logic error in syntax checker

      CHARACTER*(*) WINP, WOUT
      character wkey1*52,wkey2*64
      REAL      ROUT
      DATA      I /0/
      data wkey1/
     *'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'/
      data wkey2/
     *'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_'
     */

      KTYP=0
      KLEN=0
      WOUT(1:MAXCOL)=' '

   30 I=I+1
      IF (I.GT.MAXCOL) GO TO 90

      IF (KTYP.EQ.0) THEN
         IF (WINP(I:I).EQ.'''') THEN
            KTYP=1
            GO TO 30
         END IF
         IF (INDEX(' =,',WINP(I:I)).NE.0) GO TO 30
         IF (WINP(I:I).EQ.'#') THEN
            KTYP=3
            GO TO 9999
         END IF
         IF (WINP(I:I).EQ.'(') THEN
            KTYP=5
            GO TO 9999
         END IF
         IF (WINP(I:I).EQ.')') THEN
            KTYP=6
            GO TO 9999
         END IF
         IF (WINP(I:I).EQ.'$') THEN
            KTYP=7
            GO TO 9999
         END IF
         IF (WINP(I:I).EQ.'/') THEN
            KTYP=8
            GO TO 9999
         END IF
         IF (WINP(I:I).EQ.'.') THEN
            KTYP=10
            GO TO 91
         END IF
         IF (INDEX('0123456789+-',WINP(I:I)).NE.0) THEN
            KTYP=2
            GO TO 91
         END IF
         IF (INDEX(wkey1,WINP(I:I)).NE.0) THEN
            KTYP=9
            GO TO 91
         END IF
         KTYP=-1
         GO TO 9999
      END IF

      IF (KTYP.EQ.1) THEN
         IF (WINP(I:I).EQ.'''') GO TO 9999
         GO TO 91
      END IF

C already numbers:

      IF (KTYP.EQ.2) THEN
         IF (INDEX(' ,',WINP(I:I)).NE.0) GO TO 94
         IF (INDEX('()/',WINP(I:I)).NE.0) GO TO 93
         IF (INDEX('0123456789+-.DE',WINP(I:I)).NE.0) GO TO 91
         KTYP=-2
         GO TO 9999
      END IF

C already logical values:

      IF (KTYP.EQ.4) THEN
         IF(INDEX('TRUEFALS.',WINP(I:I)).NE.0) GO TO 91
         IF(INDEX(' ,',WINP(I:I)).NE.0) GO TO 9999
         IF (WINP(I:I).EQ.'/') GO TO 92
         KTYP=-3
         GO TO 9999
      END IF

C already a keyword:

      IF (KTYP.EQ.9) THEN
         IF (INDEX('= ,',WINP(I:I)).NE.0) GO TO 9999
         IF (INDEX('()/',WINP(I:I)).NE.0) GO TO 92
         IF (INDEX(wkey2,WINP(I:I)).NE.0) GO TO 91
         KTYP=-4
         GO TO 9999
      END IF

      IF (KTYP.EQ.10) THEN
         IF (INDEX('TF',WINP(I:I)).NE.0) THEN
            KTYP=4
            GO TO 91
         END IF
         IF (INDEX('0123456789+-DE',WINP(I:I)).NE.0) THEN
            KTYP=2
            GO TO 91
         END IF
         KTYP=-5
         GO TO 9999
      END IF

      KTYP=-9
      GO TO 9999

C  end of input line

   90 KLEN=-99
      I=0
      GO TO 9999

C  COPY CHARACTER INTO WOUT AND READ NEXT CHARACTER

   91 KLEN=KLEN+1
      WOUT(KLEN:KLEN)=WINP(I:I)
      GO TO 30

C BACKSPACE THE CARD POINTER

   92 I=I-1
      GO TO 9999

C convert literal number in WOUT into floating point number ROUT:

   93 I=I-1
   94 ROUT=0.
      READ (WOUT,'(BN,E30.0)',ERR=95) ROUT
      GO TO 9999
   95 KTYP=-2
      GO TO 9999

 9999 RETURN

      END
